/*
 * @plotMapping.java Version 1.1 05/16/2002
 *
 * Copyright (c) 2002 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;

import plot.plotWellStatusStruct;
import plot.plotMappingStruct;
import plot.plotColor;

/** Class plotMapping
 *  <p> This Plot routine will plot the Computed Volumetrics as a color map.
 *
 *  @version 1.1 05/16/2002
 *  @author  John R. Victorine
 */

public class plotMapping extends Canvas implements MouseMotionListener
{
  private plotMappingStruct        stPlot      = null;
  private plotWellStatusStruct     stSymbols   = new plotWellStatusStruct();
  private plotColor                pColor      = new plotColor();
  private plotColor                pDrawColor  = null;

  private int                      iXBegin     = 0;
  private int                      iYBegin     = 0;
  private int                      iWidth      = 0;
  private int                      iHeight     = 0;

  private int                      iTotalWells = 0;
  private int                      iX[]        = null;
  private int                      iY[]        = null;
  private int                      iZ[]        = null;
  private int                      iSymbol[]   = null;
  private int                      iTriangle[] = null;
  private int                      iWellColor  = -1;

  private double                   dPI = 3.1415927;
  private double                   dXMax = 0.0;
  private double                   dXMin = 0.0;
  private double                   dYMax = 0.0;
  private double                   dYMin = 0.0;

  private double                   dMaximum   = 0.0;
  private double                   dMinimum   = 0.0;
  private int                      iMaxColors = 768;
  private int                      iData[][];

  private double                   dValue = 0.0;
  private int                      iValue = 0;

  private String                   sZoneLabel = new String("");
  private JTextField               jTextField;
  private int                      iColor = pColor.BLUE_GREEN_RED;

  public static final int   _EXCELLENT   = 0;  // Gold   - Excellent
  public static final int   _GOOD        = 1;  // Silver - Good
  public static final int   _FAIR        = 2;  // Copper - Fair
  public static final int   _POOR        = 3;  // Lead   - Poor
  public static final int   _SELECTED    = 4;  // SELECTED

  public static final int   _HALO[][] = {
	{ 217, 217, 25 },  // Gold
	{ 230, 232, 250 }, // Silver
	{ 217, 135, 25 },  // Fair
	{ 100, 100, 100 }, // Poor
	{ 238, 130, 238 }  // Selected
//	{ 255, 105, 180 }  // Selected
//	{   0,   0, 180 }  // Selected
  };

  public static final String LEVEL[] =
  {
	"Excellent - greater than 95% accepted",
	"Good - greater than 60% accepted",
	"Fair - greater than 15% accepted",
	"Poor - tops selected",
	"Well Selected for Cross Section"
  };

  /** Constructor plotMapping()
   *  <p> This is the Constructor for this class.
   *  @param st     = Mapping Data Structure
   *  @param Width  = The Width of the Plot area
   *  @param Height = The Height of the Plot Area
   *  @param jtext  = The Text Field that will be updated with the motion of
   *                  the mouse
   */

  public plotMapping( plotMappingStruct st,
                      int Width,
                      int Height,
                      JTextField jtext )
  {
    try
    {
      stPlot     = st;
      sZoneLabel = new String(stPlot.sZone);

      jTextField = jtext;

      iWidth     = Width-iXBegin;
      iHeight    = Height-iYBegin;

      jbInit();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog((Component) null,
                                    "plotMapping(): " + e.getMessage(),
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Constructor plotMapping()
   *  <p> This is the Constructor for this class.
   *  @param st     = Mapping Data Structure
   *  @param Width  = The Width of the Plot area
   *  @param Height = The Height of the Plot Area
   *  @param jtext  = The Text Field that will be updated with the motion of
   *                  the mouse
   *  @param color  = The initial automatic color setting
   */

  public plotMapping( plotMappingStruct st,
                      int Width,
                      int Height,
                      JTextField jtext,
                      int color )
  {
    try
    {
      stPlot     = st;
      sZoneLabel = new String(stPlot.sZone);

      jTextField = jtext;

      iWidth     = Width-iXBegin;
      iHeight    = Height-iYBegin;
      iColor     = color;

      jbInit();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog((Component) null,
                                    "plotMapping(): " + e.getMessage(),
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Constructor plotMapping()    INITIALIZE CLASS ONLY
   *  <p> This is the Constructor for this class.
   */

  public plotMapping()
  {
    iWidth  = 300;
    iHeight = 500;
  }

  /** Method jbInit()
   *  <p> This method will create the Linear Plot.
   */

  public void jbInit() throws Exception
  {
    setSize(iWidth, iHeight);
    setBackground(new Color(250, 250, 250));

    // Set the Well Information

    iTotalWells = stPlot.iTotalWells;
    if (iTotalWells > 0)
    {
      setMinMaxPosition();
      setWellData();
      setWellSymbol();
    }

    setZColor();

    addMouseMotionListener(this);
  }

  /** Method close()
   * <p> This method will set the global variables to null in order to force
   *     JAVA to free the memory allocated for this panel.
   */

  public void close()
  {
    stPlot    = null;
    stSymbols = null;

    if (pColor != null)
      pColor.close();
    pColor = null;

    if (pDrawColor != null)
      pDrawColor.close();
    pDrawColor = null;

    iX         = null;
    iY         = null;
    iZ         = null;
    iSymbol    = null;
    iTriangle  = null;

    iData      = null;

    sZoneLabel = null;
    jTextField = null;
  }

  /* ------------------------------------------------------------------------ *
   * -------------------------- Set Data Methods ----------------------------
   * ------------------------------------------------------------------------ */

  /** Method setMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void setMinMaxPosition()
  {
    int    iTemp  = 0;
    int    iFound = 0;

    if ((stPlot.dXMin == 0.0) && (stPlot.dXMax == 0.0) &&
        (stPlot.dYMin == 0.0) && (stPlot.dYMax == 0.0))
    {
      for (int i=0; i<iTotalWells; i++)
      {
        if ((Math.abs(stPlot.dX[i]) > 0.0) && (Math.abs(stPlot.dY[i]) > 0.0))
        {
          if (iFound == 0)
          {
            dXMin = stPlot.dX[i];
            dXMax = dXMin;

            dYMin = stPlot.dY[i];
            dYMax = dYMin;

            iFound = 1;
          }

          if (stPlot.dX[i] < dXMin)
            dXMin = stPlot.dX[i];

          if (stPlot.dX[i] > dXMax)
            dXMax = stPlot.dX[i];

          if (stPlot.dY[i] < dYMin)
            dYMin = stPlot.dY[i];

          if (stPlot.dY[i] > dYMax)
            dYMax = stPlot.dY[i];
        }
      }

      double dX = 0.2 * (dXMax - dXMin);
      double dY = 0.2 * (dYMax - dYMin);

      dXMin = dXMin - dX;
      dXMax = dXMax + dX;
      dYMin = dYMin - dY;
      dYMax = dYMax + dY;
    }
    else
    {
      dXMin = stPlot.dXMin;
      dXMax = stPlot.dXMax;
      dYMin = stPlot.dYMin;
      dYMax = stPlot.dYMax;
    }
  }

  /** Method setWellData()
   *  <p> This method will convert the raw data to pixels so the data can be
   *      plotted.
   */

  public void setWellData()
  {
    int iRatio;
    int i;
    int j;

    iX = new int[iTotalWells];
    iY = new int[iTotalWells];

    for (i=0; i<iTotalWells; i++)
    {
      iRatio = (int)((double)iWidth *
                       (stPlot.dX[i]-dXMin) / (dXMax - dXMin));
      iX[i] = iXBegin + iRatio;
    }

    for (i=0; i<iTotalWells; i++)
    {
        iRatio = (int)((double)iHeight *
                         (stPlot.dY[i] - dYMin)/ (dYMax - dYMin));
        iY[i] = iYBegin + iHeight - iRatio;
    }
  }

  /** Method setWellSymbol()
   *  <p> This method will convert the Well Symbol to one of the recognized
   *      symbols.
   */

  private void setWellSymbol()
  {
    iSymbol   = new int[iTotalWells];
    iTriangle = new int[iTotalWells];

    for (int i=0; i<iTotalWells; i++)
    {
      iSymbol[i]   = plot.plotWellStatusStruct.NONE;
      iTriangle[i] = stPlot.iTriangle[i];

      for (int j=0; j<stSymbols.iCount; j++)
      {
        if (stPlot.status[i].equals(stSymbols.status[j][0]))
        iSymbol[i] = stSymbols.iStatus[j];
      }
    }
  }

  /** Method setZColor()
   *  <p> This method will set the color value for the Z Data Value.
   */

  public void setZColor()
  {
    int    i=0;
    int    j=0;
    double delta = 0.0;

    if ((stPlot.iRows > 0) && (stPlot.iColumns > 0))
    {
      for (i=0; i<stPlot.iRows; i++)
      {
        for (j=0; j<stPlot.iColumns; j++)
        {
          if ((i==0) && (j==0))
          {
            dMaximum = stPlot.data[i][j];
            dMinimum = stPlot.data[i][j];
          }

          if (dMaximum < stPlot.data[i][j])
            dMaximum = stPlot.data[i][j];

          if (dMinimum > stPlot.data[i][j])
            dMinimum = stPlot.data[i][j];
        }
      }

      // Initialize the Plot Color Class

      pDrawColor = new plotColor(iColor, dMinimum, dMaximum,
                                 iXBegin, iYBegin, iWidth, iHeight);
      pDrawColor.setType(stPlot.iType);
      pDrawColor.setManualData(stPlot.dLower, stPlot.dUpper, stPlot.iRGB);

      iData = new int[stPlot.iRows][stPlot.iColumns];

      for (i=0; i<stPlot.iRows; i++)
      {
        for (j=0; j<stPlot.iColumns; j++)
        {
          iData[i][j] = pDrawColor.convertDataToColor(stPlot.data[i][j]);
        }
      }
    }
  }

  /* ------------------------------------------------------------------------ *
   *  MouseListener Methods required when implementing the MouseListener Class.
   * ------------------------------------------------------------------------ */

  /** Method mouseMoved()
   *  <p> jThis method will retrieve the location of the mouse pointer and
   *       convert the postion to latitude and longitude position. It will then
   *       display the text in the Volumetric Panel.
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event)
  {
    int    iXm = event.getX();
    int    iYm = event.getY();
    int    iXo = 0;
    int    iYo = 0;
    int    iXWidth  = iWidth/(stPlot.iRows-1);
    int    iYHeight = iHeight/(stPlot.iColumns-1);
    int    iFound = 0;
    String sPosition = new String("");
    double delta = (dMaximum -  dMinimum) / (double) iMaxColors;

    if ((iXm > iXBegin) && (iXm < iXBegin + iWidth) &&
        (iYm > iYBegin) && (iYm < iYBegin + iHeight))
    {
      iXo = iXm / iXWidth;
      iYo = (iYBegin + iHeight - iYm) / iYHeight;

      dValue = stPlot.data[iXo][iYo];
      dValue = Math.ceil(dValue * 100.0);
      dValue = dValue / 100.0;

      iValue = iMaxColors-(int)((dMaximum - dValue)/delta);

      iFound = 0;
      for (int i=0; i<iTotalWells; i++)
      {
        if ((iXm > (iX[i]-5)) && (iXm < (iX[i]+5)) &&
            (iYm > (iY[i]-5)) && (iYm < (iY[i]+5)))
        {
          if (stPlot.dZ[i] > 0.0)
          {
            sPosition = new String(stPlot.sAPI[i]   + "  "    +
                                   stPlot.sLease[i] + " "     +
                                   stPlot.sWell[i]  + "     " +
                                   stPlot.sValue    + ": "    + stPlot.dZ[i]);
            dValue = stPlot.dZ[i];
          }
          else
          {
            sPosition = new String(stPlot.sAPI[i]   + "  " +
                                   stPlot.sLease[i] + " " +
                                   stPlot.sWell[i]);
          }

          iFound = 1;
          if (jTextField != null)
            jTextField.setForeground(Color.red);
        }
      }

      if (iFound == 0)
      {
        sPosition = new String(stPlot.sValue    + ": "    + dValue);
        if (jTextField != null)
          jTextField.setForeground(Color.black);
      }

      if (jTextField != null)
        jTextField.setText(sPosition);
    }
  }

  public void mouseDragged(MouseEvent event) { }
  public void mouseEntered(MouseEvent event) { }
  public void mouseExited(MouseEvent event) { }
  public void mousePressed(MouseEvent event) { }
  public void mouseReleased(MouseEvent event) { }
  public void mouseClicked(MouseEvent event){}


  /* ------------------------------------------------------------------------ *
   * ---------------------------- Draw Methods ------------------------------
   * ------------------------------------------------------------------------ */

  /** Method drawSymbolDescription()
   *  <p> This method will draw the symbols used for the mapping of the wells.
   *  @param g - Graphics pointer.
   */

  public void drawSymbolDescription(Graphics g)
  {
	int i   = 0;
    int iXg = 10;
    int iYg = 10;
    int iYh = 0;

    g.setColor(Color.black);
    g.drawString("           WELL SYMBOLS", iXg, iYg);

    for (i=0; i<stSymbols.iTotalSymbols; i++)
    {
      iXg = 10;
      iYg = 25 + i * 20;
      drawWellSymbol(g, i, iXg, iYg);
      g.drawString(" = " + stSymbols.symbols[i], iXg+20, iYg+5);
    }

    iYg = iYg + 30;

    g.setColor(Color.black);
    g.drawString(" Well Top Picks Confidence Level Halo", iXg, iYg);

    iYh = iYg + 20;

    for (i=0; i<5; i++)
    {
      iXg = 10;
      iYg = iYh + i * 25;
      drawWellSymbol(g, 0, iXg, iYg);
      drawHalo(g, i, iXg, iYg);

      g.setColor(Color.black);
      g.drawString(" = " + LEVEL[i], iXg+20, iYg+5);
	}
  }

  /** Method drawHalo()
   *  <p> This method will draw a halo around well symbol with the color
   *      representing the status of the well data.
   */

  public void drawHalo(Graphics g, int iColor, int iXp, int iYp)
  {
	if (iColor > -1)
	{
      g.setColor(new Color(_HALO[iColor][0], _HALO[iColor][1], _HALO[iColor][2]));
      g.drawOval(iXp-10, iYp-10, 20, 20);
      g.drawOval(iXp-9,  iYp-9,  18, 18);
    }
  }

  /** Method drawWellSymbol()
   *  <p> This method will draw the Well Symbol.
   *  @param g - Graphics pointer.
   *  @param iSymbol - The symbol Identifier to use in plotting the well Symbol
   *  @param iXp     - X Position of Well
   *  @param iYp     - Y Position of Well
   */

  public void drawWellSymbol(Graphics g, int iSymbol, int iXp, int iYp)
  {
    if (iWellColor == 1)
      g.setColor(Color.lightGray);
    else
      g.setColor(Color.darkGray);

    if (iWellColor == -1)
    {
      g.setColor(Color.white);
      g.fillOval(iXp-6, iYp-6, 12, 12);
    }

    switch (iSymbol)
    {
      case plot.plotWellStatusStruct.NONE:
        if (iWellColor == -1) g.setColor(Color.black);
        g.drawOval(iXp-5, iYp-5, 10, 10);
        break;

      case plot.plotWellStatusStruct.DRY:
        if (iWellColor == -1) g.setColor(Color.black);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.DRY_OIL:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-7);
        g.drawLine(iXp-5, iYp,   iXp-7, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+7);
        g.drawLine(iXp+5, iYp,   iXp+7, iYp);
        break;

      case plot.plotWellStatusStruct.DRY_GAS:
        if (iWellColor == -1) g.setColor(Color.red);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-8);
        g.drawLine(iXp-5, iYp,   iXp-8, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+8);
        g.drawLine(iXp+5, iYp,   iXp+8, iYp);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        break;

      case plot.plotWellStatusStruct.DRY_OIL_GAS:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-8);
        g.drawLine(iXp-5, iYp,   iXp-8, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+8);
        g.drawLine(iXp+5, iYp,   iXp+8, iYp);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        break;

      case plot.plotWellStatusStruct.OIL:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillOval(iXp-5, iYp-5, 10, 10);
        break;

      case plot.plotWellStatusStruct.GAS:
        if (iWellColor == -1) g.setColor(Color.red);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp+3, iYp+3, iXp+7, iYp+7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.OIL_GAS:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp+3, iYp+3, iXp+7, iYp+7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SUSPEND:
        if (iWellColor == -1) g.setColor(Color.black);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        break;

      case plot.plotWellStatusStruct.SUSPEND_OIL:
        if (iWellColor == -1) g.setColor(Color.green);
        g.drawOval(iXp-5, iYp-5, 10, 10);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SUSPEND_GAS:
        if (iWellColor == -1) g.setColor(Color.red);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SUSPEND_OIL_GAS:
        if (iWellColor == -1) g.setColor(Color.green);
        g.drawOval(iXp-5, iYp-5, 10, 10);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SWD:
        if (iWellColor == -1) g.setColor(Color.blue);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,   iYp+5);
        g.drawLine(iXp-5, iYp,   iXp+5, iYp);
        break;

      case plot.plotWellStatusStruct.OUTCROP:
        if (iWellColor == -1) g.setColor(Color.blue);
        g.drawRect(iXp-5, iYp-5, 10,10);
        break;

    }

    g.setColor(Color.black);
  }

  /** Mtehod drawTriangle()
   *  <p> This method will draw a Triangle around a well if it is to show
   *      a specific region is perforated.
   *  @param g - Graphics pointer.
   *  @param iXp     - X Position of Well
   *  @param iYp     - Y Position of Well
   */

  public void drawTriangle(Graphics g, int iTriangle, int iXp, int iYp)
  {
    if (iWellColor == 1)
      g.setColor(Color.lightGray);
    else
      g.setColor(Color.darkGray);

    if (iTriangle == 1)
    {
      g.drawLine(iXp-15, iYp+10, iXp+15, iYp+10);
      g.drawLine(iXp,    iYp-20, iXp+15, iYp+10);
      g.drawLine(iXp,    iYp-20, iXp-15, iYp+10);
    }

    g.setColor(Color.black);
  }

  /** Method drawWells()
   *  <p> This method will draw well Symbols on the panel.
   *  @param g - Graphics pointer.
   */

  public void drawWells(Graphics g)
  {
    int i, j;

    g.setColor(Color.black);

    for (i=0; i<iTotalWells; i++)
    {
      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
          if (stPlot.dZ[i] > (dMaximum - dMinimum) / 2)
            iWellColor = 1;
          else
            iWellColor = 0;

          drawWellSymbol(g, iSymbol[i], iX[i], iY[i]);
          drawTriangle(g, iTriangle[i], iX[i], iY[i]);
      }
    }
  }

  /** Method drawWellLabel()
   *  <p> This method will draw well Labels on Map.
   *  @param g - Graphics pointer.
   */

  public void drawWellLabel(Graphics g, int iLabel)
  {
    int    i, j;
    String sLabel = new String("");

    g.setColor(Color.black);

    for (i=0; i<iTotalWells; i++)
    {
      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
          if (stPlot.dZ[i] > (dMaximum - dMinimum) / 2)
            iWellColor = 1;
          else
            iWellColor = 0;

          switch (iLabel)
          {
            case 0:  // None
              sLabel = new String("");
              break;

            case 1:  // Order Number
              sLabel = new String("" + i);
              break;

            case 2:  // API Number
              sLabel = new String(stPlot.sAPI[i]);
              break;

            case 3:  // Lease Name
              sLabel = new String(stPlot.sLease[i] + " " + stPlot.sWell[i]);
              break;
          }

          if (iWellColor == 1)
            g.setColor(Color.lightGray);
          else
            g.setColor(Color.darkGray);

          g.drawString(sLabel, iX[i]+12 , iY[i]);
      }
    }
  }

  /** Method drawGrid()
   *  <p> This method will create a grid.
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
    int    iXo = 0;
    int    iYo = 0;
    int    iXWidth  = iWidth/(stPlot.iRows-1);
    int    iYHeight = iHeight/(stPlot.iColumns-1);

    g.setColor(Color.lightGray);
    for (int i=0; i<stPlot.iRows; i++)
    {
      iXo = iXBegin + i * iXWidth;

      for (int j=0; j<stPlot.iColumns; j++)
      {
        iYo = iYBegin + iHeight - (j+1) * iYHeight;
        g.drawRect(iXo, iYo, iXWidth, iYHeight);
      }
    }
  }

  /** Method drawSelection()
   *  <p> This method will draw a line on the Color Scale of the value of
   *      the selected cell.
   *  @param g - Graphics Handle;
   */

  public void drawSelection(Graphics g)
  {
    int    iScale = iMaxColors / (iHeight-51);
    int    i = iValue * iScale;

    if (i>iMaxColors)
      i = iMaxColors;

    g.setColor(Color.red);
    g.drawLine(iXBegin+iWidth+50,  iYBegin+iHeight-i,
               iXBegin+iWidth+100, iYBegin+iHeight-i);
  }

  /** Method drawData()
   *  <p> This method will paint the Volumetrics values.
   *  @param g - Graphics Handle;
   */

  public void drawData(Graphics g)
  {
    int    iXo = 0;
    int    iYo = 0;
    int    iXWidth  = iWidth/(stPlot.iRows-1);
    int    iYHeight = iHeight/(stPlot.iColumns-1);
    int    iValue = 0;
    int    iRed   = 0;
    int    iGreen = 0;
    int    iBlue  = 0;

    for (int i=0; i<stPlot.iRows; i++)
    {
      iXo = iXBegin + i * iXWidth;

      for (int j=0; j<stPlot.iColumns; j++)
      {
        iYo = iYBegin + iHeight - (j+1) * iYHeight;

        iValue = 768 - iData[i][j];

        pDrawColor.computeColor(iValue);

        g.setColor(new Color(pDrawColor.getRed(),
                             pDrawColor.getGreen(),
                             pDrawColor.getBlue()));
        g.fillRect(iXo, iYo, iXWidth, iYHeight);
      }
    }
  }

  /** Method drawMap()
   *  <p> This method will paint the volumetrics data.
   *  @param g - Graphics Handle;
   */

  public void drawMap(Graphics g)
  {
    drawData(g);
    drawGrid(g);
    drawWells(g);

    if (stPlot.iType == 1)
    {
      pDrawColor.setMinMaxValue(stPlot.dLower[4], stPlot.dUpper[0]);
      pDrawColor.setManualData(stPlot.dLower, stPlot.dUpper, stPlot.iRGB);
    }

    pDrawColor.drawScale(g);
  }

  /** Method paint()
   *  <p> This method will paint the volumetrics data.
   *  @param g - Graphics Handle;
   */

  public void paint(Graphics g)
  {
    drawMap(g);
  }
}
