/*
 * @plotXYData.java Version 1.1 07/14/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.awt.event.*;

import plot.plotXYStruct;
import plot.plotSymbolStruct;
import plot.plotSymbol;

import iqstrat.iqstratHeadersListStruct;
import plot.plotWellStatusStruct;
import plot.plotMapping;

import util.utilLandGridListStruct;      // TRS Land Grid List Structure
import util.utilPlotLandGrid;

import quake.quakeSensorsStruct;         // Sensor Data Structure

/** Class plotXY
 *  <p> This class will create a generic X-Y Plot for an array of data
 *
 *  @version 1.1 07/14/2011
 *  @author  John R. Victorine
 */

public class plotXYData extends Canvas
{
  public static final int _OFF   = 0;
  public static final int _ON    = 1;

  private static final int COUNTY_GRID = util.utilPlotLandGrid.COUNTY_GRID;
  private static final int AREA_GRID   = util.utilPlotLandGrid.AREA_GRID;
  private static final int NO_TEXT     = util.utilPlotLandGrid.NO_TEXT;

  private int                      iAction   = _ON;
  private int                      iTRS      = _ON;

  private int                      iPlot     = -1;
  private plotXYStruct             stPlot    = null;

  private iqstratHeadersListStruct stWells   = null;

  private int                      iGrid     = AREA_GRID;
  private utilLandGridListStruct   stGrid    = null; // TRS Land Grid List Structure
  private utilPlotLandGrid         pGrid     = null;

  private plotMapping              plotMap   = new plotMapping();
  private plotWellStatusStruct     stSymbols = new plotWellStatusStruct();

  private quakeSensorsStruct       stSensor = null; // CO2 Sensors Location Structure

  // Plot Dimensions

  private int              iXBegin = 80;     // Left Edge of Plot Area
  private int              iYBegin = 60;     // Top Edge of Plot Area
  private int              iWidth  = 500; //300;    // Width of Plot Area
  private int              iHeight = 500; //300;    // Height of Plot Area

  // Plot Data

  private int              iRows = 0;  // Total Number of Rows
  private int              iX[];       // Array of X Data converted to Pixels
  private int              iY[];       // Array of Y Data converted to Pixels
  private plotSymbolStruct stSymbol[]; // Array of Symbol and Color for Point

  // Well Plot Data

  private int              iWRows     = 0;
  private int              iWX[]      = null;
  private int              iWY[]      = null;
  private int              iWSymbol[] = null;

  // Sensor Plot Data

  private int              iSRows     = 0;
  private int              iSX[]      = null;
  private int              iSY[]      = null;

  // Legend

  private int              iLegend = 0; // Total Number of Legends
  private plotSymbolStruct stLegend[];  // Array of Symbol and Color for Legend

  // Plot Symbols

  private plotSymbol pSymbol = new plotSymbol();

  // Filter Extra Data Indicator

  private boolean bFlag = false;

  /** Constructor plotXYData()
   *  <p> This is the Constructor for this class.
   *  @param iPlot   = the Plot Type
   *  @param stPlot  = Plot Data Structure.
   *  @param stWells = SWD Wells & Wellington KGS Wells
   */

  public plotXYData(int iPlot, plotXYStruct stPlot,
                    iqstratHeadersListStruct stWells )
  {
    setBackground(Color.white);

    this.iPlot   = iPlot;
    this.stPlot  = stPlot;
    this.stWells = stWells;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    iRows = stPlot.iRows;

    if (stPlot.stSymbols != null)
      this.stSymbol = stPlot.stSymbols;

    this.iLegend = stPlot.iLegend;
    if (stPlot.stLegend != null)
      this.stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();
  }

  /** Constructor plotXYData()
   *  <p> This is the Constructor for this class.
   *  @param iPlot    = the Plot Type
   *  @param stPlot   = Plot Data Structure.
   *  @param stWells  = SWD Wells & Wellington KGS Wells
   *  @param stSensor = CO2 Sensor Location Structure
   *  @param stGrid   = TRS Land Grid List Structure
   */

  public plotXYData(int iPlot, plotXYStruct  stPlot,
                    iqstratHeadersListStruct stWells,
                    quakeSensorsStruct       stSensor,
                    utilLandGridListStruct   stGrid )
  {
    setBackground(Color.white);

    this.iPlot    = iPlot;
    this.stPlot   = stPlot;
    this.stWells  = stWells;
    this.stSensor = stSensor;
    this.stGrid   = stGrid;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    iRows = stPlot.iRows;

    if (stPlot.stSymbols != null)
      this.stSymbol = stPlot.stSymbols;

    this.iLegend = stPlot.iLegend;
    if (stPlot.stLegend != null)
      this.stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();

    if (stGrid != null)
      pGrid = new utilPlotLandGrid( stGrid,
                                    iXBegin,
                                    iYBegin,
                                    iWidth  + iXBegin,
                                    iHeight + iYBegin,
                                    stPlot.dXMinimum, stPlot.dXMaximum,
                                    stPlot.dYMinimum, stPlot.dYMaximum,
                                    iGrid );
  }

  /* =============================================================== *
   * ----------------------- DELETE METHODS ------------------------ *
   * =============================================================== */

  /** Method close()
   * <p> This method will set strings and arrays to null so Java can
   *     automatically clear memory
   */

  public void close()
  {
    stPlot    = null;
    stWells   = null;
    stSymbols = null;
    stSensor  = null;
    stGrid    = null;

    iX        = null; // Array of X Data converted to Pixels
    iY        = null; // Array of Y Data converted to Pixels
    stSymbol  = null; // Array of Symbol and Color for Point
    stLegend  = null; // Array of Symbol and Color for Legend

    iWX       = null;
    iWY       = null;
    iWSymbol  = null;

    iSX       = null;
    iSY       = null;

    if (pSymbol != null)
      pSymbol.close();
    pSymbol  = null;

    if (plotMap != null)
      plotMap.close();
    plotMap = null;
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setData(plotXYStruct stPlot)
   * <p> This method will change the Plot XY Data Structure
   * @param stPlot = the Plot XY Data Structure
   */

  public void setData(plotXYStruct stPlot)
  {
    this.stPlot = stPlot;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    iRows = stPlot.iRows;

    if (stPlot.stSymbols != null)
      this.stSymbol = stPlot.stSymbols;

    this.iLegend = stPlot.iLegend;
    if (stPlot.stLegend != null)
      this.stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();

    if (pGrid != null)
      pGrid.setXYLimt(stPlot.dXMinimum, stPlot.dXMaximum,
                      stPlot.dYMinimum, stPlot.dYMaximum );

    repaint();
  }

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    this.iLegend  = iLegend;
    this.stLegend = stLegend;
  }

  /** Method setLegendFlag()
   * <p> This method will set the Legend on or off by toggling the iLegend
   *     value
   * @param iLegend = The total number of legend values
   */

  public void setLegendFlag(int iLegend)
  {
    this.iLegend = iLegend;
    repaint();
  }

  /** Method setFluid()
   *  <p> This method will set the Fluid type and recompute the marker minerals
   */

  public void setFluid( int iFluid )
  {

  }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
    this.stPlot.sTitle1 = new String(sText);
    repaint();
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
    this.stPlot.sTitle2 = new String(sText);
    repaint();
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
    this.stPlot.sTitle3 = new String(sText);
    repaint();
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sXAxis      = new String(sText);
    this.stPlot.iXLog       = iGrid;
    this.stPlot.dXMaximum   = dMax;
    this.stPlot.dXMinimum   = dMin;
    this.stPlot.dXIncrement = dInc;
    this.stPlot.iXCycles    = iCycle;

    convertDataToPixels();

    if (pGrid != null)
      pGrid.setXYLimt(stPlot.dXMinimum, stPlot.dXMaximum,
                      stPlot.dYMinimum, stPlot.dYMaximum );
    repaint();
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sYAxis      = new String(sText);
    this.stPlot.iYLog       = iGrid;
    this.stPlot.dYMaximum   = dMax;
    this.stPlot.dYMinimum   = dMin;
    this.stPlot.dYIncrement = dInc;
    this.stPlot.iYCycles    = iCycle;

    convertDataToPixels();

    if (pGrid != null)
      pGrid.setXYLimt(stPlot.dXMinimum, stPlot.dXMaximum,
                      stPlot.dYMinimum, stPlot.dYMaximum );
    repaint();
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag)
  {
    this.bFlag = bFlag;
    repaint();
  }

  /** Method setTriangle()
   * <p> This method will set the triangle data
   * @param iData      = The total number points
   * @param sLithology = The lithology values
   */

  public void setTriangle(int iData, String sLithology[])
  {

  }

  /** Method setStatus()
   *  <p> Turn the Wells on or off.
   *  @param iaction = 0 wells off, 1 wells on.
   */

  public void setStatus(int iAction)
  {
    this.iAction = iAction;
    repaint();
  }

  /** Method setGrid()
   *  <p> Turn the TRS Grid on or off.
   *  @param iTRS = 0 wells off, 1 wells on.
   */

  public void setGrid(int iTRS)
  {
	this.iTRS = iTRS;
	repaint();
  }

  /* ================================================================== *
   * ------------------------ SUPPORT METHODS ------------------------- *
   * ================================================================== */

  /** Method void convertDataToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertDataToPixels()
  {
    int    i      = 0;
    int    iRatio = 0;
    double dXMin  = 0;
    double dXMax  = 0;
    double dYMin  = 0;
    double dYMax  = 0;

    iRows = stPlot.iRows;
    iX    = new int [iRows];
    iY    = new int [iRows];

    // Set the X-Axis Data

    // IF the X-Axis Data is LINEAR THEN
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
    {
      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iWidth *
                       (stPlot.dX[i] - stPlot.dXMinimum) /
                       (stPlot.dXMaximum - stPlot.dXMinimum));
        iX[i] = iXBegin + iRatio;
      }
    }
    // IF the X-Axis Data is LOG THEN
    else
    {
      dXMin = Math.log(stPlot.dXMinimum) / Math.log(10.0);
      dXMax = Math.log(stPlot.dXMaximum) / Math.log(10.0);

      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iWidth *
                       ((Math.log(stPlot.dX[i])/Math.log(10.0))-dXMin) /
                       (dXMax - dXMin));
        iX[i] = iXBegin + iRatio;
      }
    }

    // Set the Y-Axis Data

    // IF the Y-Axis Data is LINEAR THEN
    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
    {
      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iHeight *
                       (stPlot.dY[i] - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iY[i] = iHeight + iYBegin - iRatio;
      }
    }
    // IF the Y-Axis Data is LOG THEN
    else
    {
      dYMin = Math.log(stPlot.dYMinimum) / Math.log(10.0);
      dYMax = Math.log(stPlot.dYMaximum) / Math.log(10.0);

      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iHeight *
                       ((Math.log(stPlot.dY[i])/Math.log(10.0))-dYMin) /
                       (dYMax - dYMin));
        iY[i] = iHeight + iYBegin - iRatio;
      }
    }

    convertWellDataToPixels();
    convertSensorsToPixels();
  }

  /** Method void convertWellDataToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertWellDataToPixels()
  {
    int    iRatio     = 0;
    int    i          = 0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;

    if (stWells != null)
    {
      iWRows   = stWells.iCount;
      iWX      = new int[iWRows];
      iWY      = new int[iWRows];
      iWSymbol = new int[iWRows];

      for (i=0; i<iWRows; i++)
      {
        dLongitude = stWells.stItem[i].dLongitude;
//      iRatio = (int)((double)iWidth * (dLongitude-dXMin) / (dXMax - dXMin));
//      iX[i] = iXBegin + iRatio;
        iRatio = (int)((double)iWidth *
                       (dLongitude - stPlot.dXMinimum) /
                       (stPlot.dXMaximum - stPlot.dXMinimum));
        iWX[i] = iXBegin + iRatio;
      }

      for (i=0; i<iWRows; i++)
      {
        dLatitude = stWells.stItem[i].dLatitude;
//      iRatio = (int)((double)iHeight * (dLatitude - dYMin)/ (dYMax - dYMin));
//      iY[i] = iYBegin + iHeight - iRatio;
        iRatio = (int)((double)iHeight *
                       (dLatitude - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iWY[i] = iHeight + iYBegin - iRatio;
	  }

	  getWellSymbol();
    }
  }

  /** Method getWellSymbol()
   *  <p> This method will convert the Well Symbol to one of the recognized
   *      symbols.
   */

  private void getWellSymbol()
  {
    for (int i=0; i<iWRows; i++)
    {
      iWSymbol[i] = plot.plotWellStatusStruct.NONE;

      for (int j=0; j<stSymbols.iCount; j++)
      {
        if (stWells.stItem[i].status.equals(stSymbols.status[j][0]))
          iWSymbol[i] = stSymbols.iStatus[j];
      }
    }
  }

  /** Method void convertSensorsToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertSensorsToPixels()
  {
    int    iRatio     = 0;
    int    i          = 0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;

    if (stSensor != null)
    {
      iSRows   = stSensor.iTotal;
      iSX      = new int[iSRows];
      iSY      = new int[iSRows];

      for (i=0; i<iSRows; i++)
      {
        dLongitude = stSensor.dLong[i];
        iRatio = (int)((double)iWidth *
                       (dLongitude - stPlot.dXMinimum) /
                       (stPlot.dXMaximum - stPlot.dXMinimum));
        iSX[i] = iXBegin + iRatio;
      }

      for (i=0; i<iSRows; i++)
      {
        dLatitude = stSensor.dLat[i];
        iRatio = (int)((double)iHeight *
                       (dLatitude - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iSY[i] = iHeight + iYBegin - iRatio;
	  }
    }
  }

  /* ================================================================== *
   * -------------------------- DRAW METHODS -------------------------- *
   * ================================================================== */

  /** Method drawTitles()
   *  <p> This method will draw the titles of the Plot.
   *  @param g - Graphics pointer.
   */

  public void drawTitles(Graphics g)
  {
    int iTemp = 0;

    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 14) );

    if (stPlot.sTitle1 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle1.length())));
      g.drawString(stPlot.sTitle1, iTemp / 2, 10);
    }

    if (stPlot.sTitle2 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle2.length())));
      g.drawString(stPlot.sTitle2, iTemp / 2, 30);
    }

    if (stPlot.sTitle3 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle3.length())));
      g.drawString(stPlot.sTitle3, iTemp / 2, 50);
    }
  }

  /** Method drawLabels()
   *  <p> This method will draw the Linear Plot Labels.
   *  @param g - Graphics pointer.
   */

  public void drawLabels(Graphics g)
  {
    int ivalue;

    g.setColor(Color.black);

    if (stPlot.sXAxis != null)
    {
      ivalue = (iWidth+100)/2 - 3 * (stPlot.sXAxis).length();
      g.drawString(stPlot.sXAxis, ivalue, iYBegin+iHeight+35);
    }

    if (stPlot.sYAxis != null)
    {
      ivalue = (iHeight + 100) / 2 - 3 * (stPlot.sYAxis).length();

      Graphics2D g2 = (Graphics2D) g;
      g2.rotate( (Math.PI / 2.0));
      g2.drawString(stPlot.sYAxis, ivalue, -10);
      g2.rotate( -1.0 * (Math.PI / 2.0));
    }
  }

  /** Method drawGrid()
   *  <p> This method will draw the Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
      drawLinearXAxisGrid(g);
    else
      drawLogXAxisGrid(g);

    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
      drawLinearYAxisGrid(g);
    else
      drawLogYAxisGrid(g);
  }

  /** Method drawLinearXAxisGrid()
   *  <p> This method will draw the Linear X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLinearXAxisGrid(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    double dx     = 0.0;
    double del    = 0.0;

    iXinc = (int) ((double)iWidth * stPlot.dXIncrement /
                   (stPlot.dXMaximum - stPlot.dXMinimum));

    idraw = iXBegin;
    while (idraw < iXBegin+iWidth+5)
    {
      dx     = stPlot.dXMinimum + (double) i * stPlot.dXIncrement;
      iRatio = (int) ((double) iWidth *
                      (dx - stPlot.dXMinimum) /
                      (stPlot.dXMaximum - stPlot.dXMinimum));
      idraw = iXBegin + iRatio;

      if (idraw < iXBegin+iWidth+5)
      {
        g.setColor(Color.darkGray);
        if (iTRS == _ON)
          g.setColor(new Color(200,200,200));
        g.drawLine(idraw, iYBegin, idraw, iYBegin+iHeight);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );
        dx = Math.round(dx*1000)/1000.0;
        g.drawString(""+dx, idraw, iYBegin + iHeight + 15);

        for (j=1; j<5; j++)
        {
          idel = idraw + (j * iXinc) / 5;
          if (idel < iXBegin+iWidth)
          {
            g.setColor(Color.lightGray);
            if (iTRS == _OFF)
              g.drawLine(idel, iYBegin, idel, iYBegin+iHeight);
          }
        }
      }

      i++;
    }
  }

  /** Method drawLogXAxisGrid()
   *  <p> This method will draw the Log X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLogXAxisGrid(Graphics g)
  {
    int    i, j;
    int    l=1;
    int    k=0;
    int    iXinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;

    iXinc = 1 + iWidth / stPlot.iXCycles;

    g.setColor(Color.darkGray);
    g.drawLine(iXBegin, iYBegin, iXBegin, iYBegin+iHeight);

    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);
      g.drawLine(i, iYBegin, i, iYBegin+iHeight);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iXinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - iXinc + (int) dTemp;

        g.drawLine(jXinc, iYBegin, jXinc, iYBegin+iHeight);
      }
    }

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    del = stPlot.dXIncrement;
    g.drawString("" + stPlot.dXMinimum, iXBegin-10, iYBegin+iHeight+15);
    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);

      dLabel = (double) k * 10.0 * stPlot.dXMinimum;
      k = k * 10;

      g.setColor(Color.black);
      g.drawString("" + dLabel, i-10, iYBegin+iHeight+15);
      del = stPlot.dXIncrement + del;
    }
  }

  /** Method drawLinearYAxisGrid()
   *  <p> This method will draw the Linear Y-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLinearYAxisGrid(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iYinc  = 0;
    double dy     = 0.0;
    double del    = 0.0;

    iYinc = (int) ((double)iHeight * stPlot.dYIncrement /
                   (stPlot.dYMaximum - stPlot.dYMinimum));

    idraw = iYBegin + iHeight;
    while (idraw > iYBegin-5)
    {
      dy = stPlot.dYMinimum + (double) i * stPlot.dYIncrement;
      iRatio = (int)((double)iHeight *
                     (dy - stPlot.dYMinimum) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
      idraw = iHeight + iYBegin - iRatio;

      if (idraw > iYBegin-5)
      {
        g.setColor(Color.darkGray);
        if (iTRS == _ON)
          g.setColor(new Color(200,200,200));

        g.drawLine(iXBegin, idraw, iXBegin+iWidth, idraw);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );
        dy = Math.round(dy*1000)/1000.0;
        g.drawString("" + dy, iXBegin-45, idraw);

        for (j=1; j<5; j++)
        {
          idel = idraw - (j * iYinc) / 5;
          if (idel > iYBegin)
          {
            g.setColor(Color.lightGray);
            if (iTRS == _OFF)
              g.drawLine(iXBegin, idel, iXBegin+iWidth, idel);
          }
        }
      }

      i++;
    }
  }

  /** Method drawLogYAxisGrid()
   *  <p> This method will draw the Log X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawLogYAxisGrid(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    k = 1;
    int    iYinc  = 0;
    int    jXinc  = 0;
    double dy     = 0.0;
    double dTemp  = 0.0;
    double dXinc  = 0.0;

    iYinc = 1 + iHeight / stPlot.iYCycles;

    for (i=iYBegin+iHeight; i>iYBegin; i-=iYinc)
    {
      k = i;
      g.setColor(Color.darkGray);
      g.drawLine(iXBegin, i, iXBegin+iWidth, i);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iYinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - (int) dTemp;

        g.drawLine(iXBegin, jXinc, iXBegin+iWidth, jXinc);
      }
    }

    g.setColor(Color.darkGray);
    g.drawLine(iXBegin, k-iYinc, iXBegin+iWidth, k-iYinc);

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    g.drawString("" + stPlot.dYMinimum, iXBegin-45, iYBegin+iHeight);
    for (i=iYBegin+iHeight; i>iYBegin; i-=iYinc)
    {
      dy = (double) k * 10.0 * stPlot.dYMinimum;
      k = k * 10;
      g.drawString("" + dy, iXBegin-45, i-iYinc);
    }
  }

  /** Method drawPlotData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawPlotData(Graphics g)
  {
    int iDraw = 0;
    int iX1   = 0;
    int iX2   = 0;
    int iY1   = 0;
    int iY2   = 0;

    for (int i=0; i<iRows; i++)
    {
      iX1 = iX[i];
      iY1 = iY[i];

      if ( (iX1 > iXBegin) && (iX1 < iXBegin + iWidth)  &&
           (iY1 > iYBegin) && (iY1 < iYBegin + iHeight))
      {
        iDraw = 1;

        if (iDraw == 1)
        {
          pSymbol.drawSymbol(g, stSymbol[i], iX[i], iY[i]);
        }
      }
    }
  }

  /** Method drawLegend()
   *  <p> This method will create the Legend
   *  @param g        = Graphics pointer.
   */

  public void drawLegend(Graphics g)
  {
    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    if (stPlot.sLegTitle.length() > 0)
      g.drawString(stPlot.sLegTitle, iXBegin+iWidth+10, iYBegin-13);

    for (int i=0; i<iLegend; i++)
    {
      pSymbol.drawSymbol(g, stLegend[i], iXBegin+iWidth+10, iYBegin+i*13);

      g.setColor(Color.black);
//      g.setColor(new Color(stLegend[i].iRed,
//                           stLegend[i].iGreen,
//                           stLegend[i].iBlue));
      g.drawString(stLegend[i].symbol, iXBegin+iWidth+10+8, iYBegin+i*13);
    }

    g.setColor(Color.black);
    g.drawString("USGS", iXBegin+iWidth+10+8, iYBegin+iLegend*13+13);
    pSymbol.drawSymbol(g,
                       plot.plotSymbolStruct.FILLED,
                       plot.plotSymbolStruct.DIAMOND,
                       iXBegin+iWidth+10, iYBegin+iLegend*13+13);

    g.drawString("CO2 Sensor Array", iXBegin+iWidth+10+8, iYBegin+iLegend*13+26);
    pSymbol.drawSymbol(g,
                       plot.plotSymbolStruct.FILLED,
                       plot.plotSymbolStruct.TRIANGLE,
                       iXBegin+iWidth+10, iYBegin+iLegend*13+26);

    g.drawString("CO2 Sensor Array > 5 km", iXBegin+iWidth+10+8, iYBegin+iLegend*13+39);
    pSymbol.drawSymbol(g,
                       plot.plotSymbolStruct.EMPTY,
                       plot.plotSymbolStruct.TRIANGLE,
                       iXBegin+iWidth+10, iYBegin+iLegend*13+39);
    g.drawString("* Distances are measured with", iXBegin+iWidth+10, iYBegin+iLegend*13+52);
    g.drawString("respect to the KGS 2-32",     iXBegin+iWidth+10, iYBegin+iLegend*13+65);
    g.drawString("CO2-EOR injection well,",     iXBegin+iWidth+10, iYBegin+iLegend*13+78);
    g.drawString("located in 32-31S-1W.",       iXBegin+iWidth+10, iYBegin+iLegend*13+91);

    plotMap.drawWellSymbol(g, plot.plotWellStatusStruct.NONE, iXBegin+iWidth+10, iYBegin+iLegend*13+117);
    g.drawString("Wellington KGS Wells", iXBegin+iWidth+10+8, iYBegin+iLegend*13+120);

    plotMap.drawWellSymbol(g, plot.plotWellStatusStruct.SWD, iXBegin+iWidth+10, iYBegin+iLegend*13+133);
    g.drawString("Salt Water Disposal Wells", iXBegin+iWidth+10+8, iYBegin+iLegend*13+136);

    g.setColor(new Color(165, 42, 42));
    g.fillRect(iXBegin+iWidth+5, iYBegin+iLegend*13+144, 10, 10);
    g.setColor(Color.black);
    g.drawString("CO2 Seismic Sensors", iXBegin+iWidth+10+8, iYBegin+iLegend*13+151);

    g.setColor(Color.black);
    g.drawString("(*) Accuracy in depth and", iXBegin+iWidth+10, iYBegin+iHeight+20-52);
    g.drawString("location is significantly", iXBegin+iWidth+10, iYBegin+iHeight+20-39);
    g.drawString("decreased at distances",    iXBegin+iWidth+10, iYBegin+iHeight+20-26);
    g.drawString("greater than 5km from the", iXBegin+iWidth+10, iYBegin+iHeight+20-13);
    g.drawString("CO2 array.",                iXBegin+iWidth+10, iYBegin+iHeight+20);
  }

  /** Method drawWells()
   *  <p> This method will draw the Linear Plot Curves.
   *  @param g - Graphics pointer.
   */

  public void drawWells(Graphics g)
  {
    for (int i=0; i<iWRows; i++)
    {
      if ((iWX[i] >= iXBegin) && (iWX[i] <= iXBegin+iWidth) &&
          (iWY[i] >= iYBegin) && (iWY[i] <= iYBegin+iHeight))
      {
        plotMap.drawWellSymbol(g, iWSymbol[i], iWX[i], iWY[i]);
	  }
	}
  }

  /** Method drawSensors()
   *  <p> This method will draw the Linear Plot Curves.
   *  @param g - Graphics pointer.
   */

  public void drawSensors(Graphics g)
  {
    int    iTotal    = 10;
    int    iOrder[]  = {14,13,10, 8, 5, 4, 2, 6, 7,15};
    int    ix[]      = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    int    iy[]      = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    int    iContinue = 1;

    for (int i=0; i<iSRows; i++)
    {
      if ((iSX[i] >= iXBegin) && (iSX[i] <= iXBegin+iWidth) &&
          (iSY[i] >= iYBegin) && (iSY[i] <= iYBegin+iHeight))
      {
//        g.setColor(new Color(210, 180, 140));
        g.setColor(new Color(165, 42, 42));
        g.fillRect(iSX[i]-5, iSY[i]-5, 10, 10);
	  }
	}
  }

  /** Method draw()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void draw(Graphics g)
  {
    if ((iTRS == _ON) && (pGrid != null))
    {
      pGrid.draw(g);
      pGrid.drawMajorSTRLabel(g);
	}
    else
    {
      drawLabels(g);
      drawGrid(g);
	}

    drawTitles(g);

    g.setColor(Color.black);
    if (stPlot.sLegRange.length() > 0)
      g.drawString(stPlot.sLegRange, iXBegin+iWidth+10, iYBegin-26);
    if (stPlot.bLegend) drawLegend(g);

    drawSensors(g);
    if (iAction == _ON) { drawWells(g); }

	drawPlotData(g);
  }

  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
	draw(g);
  }
}