/*
 * @plotXYZData.java Version 1.1 09/20/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.awt.event.*;

import plot.plotXYStruct;
import plot.plotSymbolStruct;
import plot.plotSymbol;

import iqstrat.iqstratHeadersListStruct;
import plot.plotWellStatusStruct;
import plot.plotMapping;

import util.utilLandGridListStruct;      // TRS Land Grid List Structure
import util.utilPlotLandGrid;

import quake.quakeSensorsStruct;         // Sensor Data Structure

/** Class plotXY
 *  <p> This class will create a generic X-Y Plot for an array of data
 *
 *  @version 1.1 09/20/2011
 *  @author  John R. Victorine
 */

public class plotXYZData extends Canvas
{
  private static final int _DATA = 0;
  private static final int _WELL = 1;

  public static final int _OFF   = 0;
  public static final int _ON    = 1;

  private static final int COUNTY_GRID = util.utilPlotLandGrid.COUNTY_GRID;
  private static final int AREA_GRID   = util.utilPlotLandGrid.AREA_GRID;
  private static final int NO_TEXT     = util.utilPlotLandGrid.NO_TEXT;

  private int              iAction = _ON;
  private int              iTRS    = _ON;

  private int              iPlot   = -1;
  private plotXYStruct     stPlot  = null;
  private plotSymbol       pSymbol = new plotSymbol();

  private iqstratHeadersListStruct stWells   = null;

  private int                      iGrid     = AREA_GRID;
  private utilLandGridListStruct   stGrid    = null; // TRS Land Grid List Structure
  private utilPlotLandGrid         pGrid     = null;

  private plotMapping              plotMap   = new plotMapping();
  private plotWellStatusStruct     stSymbols = new plotWellStatusStruct();

  private quakeSensorsStruct       stSensor = null; // CO2 Sensors Location Structure

  // Plot Data

  private int              iRows = 0;          // Total Number of Rows
  private int              iX[]  = null;       // Array of Y Data converted to Pixels
  private int              iY[]  = null;       // Array of Z Data converted to Pixels
  private plotSymbolStruct stSymbol[]  = null; // Array of Symbol and Color for Point
  private int              iXp[] = null; // Array of X Data converted to Pixels for 2D Start
  private int              iYp[] = null; // Array of X Data converted to Pixels for 2D Start

  // Well Plot Data

  private int              iWRows     = 0;
  private int              iWX[]      = null;
  private int              iWY[]      = null;
  private int              iWXp[]     = null; // Array of X Data converted to Pixels for 2D Start
  private int              iWYp[]     = null; // Array of X Data converted to Pixels for 2D Start
  private int              iWTD[]     = null;
  private int              iWSymbol[] = null;

  // Sensor Plot Data

  private int              iSRows     = 0;
  private int              iSX[]      = null;
  private int              iSY[]      = null;
  private int              iSXp[]     = null; // Array of X Data converted to Pixels for 2D Start
  private int              iSYp[]     = null; // Array of X Data converted to Pixels for 2D Start
  private int              iSTD[]     = null;

  // Order of Plot

                                              //              well or data list
  private int              ipData[][] = null; // [ # in data structure ] [ identifier ]
  private int              iTotal     = 0;

  // Legend

  private int              iLegend = 0; // Total Number of Legends
  private plotSymbolStruct stLegend[];  // Array of Symbol and Color for Legend

  // Filter Extra Data Indicator

  private boolean bFlag = false;

  // Plot Parameters

  private int iWidth  = 700; // Width of Plot Area
  private int iHeight = 700; // Height of Plot Area

  private int iXBegin = 80;  // Left Edge of Plot Area
  private int iYBegin = 80;//60;  // Top Edge of Plot Area
  private int iYBase  = 40;  // Height in pixels for Y-Axis Labels

  private int iWo     = 350; //300; // Width of 2D Plot Area
  private int iHo     = 300; // Height of 2D Plot Area

  private int iXo     = 0;   // X-Coordinate of Top Front of 3D Plot Area
  private int iYo     = 0;   // Y-Coordinate of Top Front of 3D Plot Area
  private int iX1     = 0;   // X-Coordinate of Top Back of 3D Plot Area
  private int iY1     = 0;   // Y-Coordinate of Top Back of 3D Plot Area

  // Rotate Drawing Area

  private int iXMove  = 0; //-100;
  private int iYMove  = 0; //100;

/***********************************************************************************
 * ------------------------- Visual of 3D Plot Parameters ------------------------ *
 ***********************************************************************************
 *
 *   -|<------------- iWidth ----------------->|
 *    |           (iX1,iY1)
 *    |     *         *------------------------  ---  Each YZ plot area position
 *   iXBegin,iYBegin .|                        |  |   is based on the X value
 *    |             . |                        |  |   The iXp, iYp will be found
 *    |            .  |     gridXZ             |  |   by interpolation
 *    | (iXp,iYp) *   |                        | iHo
 *    |          . Y  |                        |  |
 *    |         .     |                        |  |
 *    |        .      |                        |  |
 *    |       .gridYZ .------------------------. ---
 *  iHeight (iXo,iYo).|<-------- iWo --------->|
 *    |     *------------------------        .
 *    |     |      .                 | gridXY
 *    |     |     .                  |     .
 *    |     |    .                   |    .
 *    |     |   .                    | Z .
 *    |     |  .                     |  .
 *    |     | .                      | .
 *    |     |.                       |.
 *    | --- .------------------------.
 *    | iYBase          X
 *   --- -----
 *
 ************************************************************************************/

  /** Constructor plotXYZData()
   *  <p> This is the Constructor for this class.
   *  @param iPlot  = the Plot Type
   *  @param stPlot = Plot Data Structure.
   */

  public plotXYZData( int iPlot, plotXYStruct stPlot,
                      iqstratHeadersListStruct stWells )
  {
    setBackground(Color.white);

    this.iPlot   = iPlot;
    this.stPlot  = stPlot;
    this.stWells = stWells;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    if (stPlot.iWo != 0)
      iWo = stPlot.iWo;

    if (stPlot.iHo != 0)
      iHo = stPlot.iHo;

    iXo = iXBegin;
    iYo = iHeight - iHo - iYBegin - iYBase;

    iX1 = iWidth  - iWo - iXBegin + iXMove;
//    iX1 = iWidth  - iWo - iXBegin;
    iY1 = iYBegin + iYMove;

    iRows = stPlot.iRows;

    if (stPlot.stSymbols != null)
      this.stSymbol = stPlot.stSymbols;

    this.iLegend = stPlot.iLegend;
    if (stPlot.stLegend != null)
      this.stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();
  }

  /** Constructor plotXYZData()
   *  <p> This is the Constructor for this class.
   *  @param iPlot    = the Plot Type
   *  @param stPlot   = Plot Data Structure.
   *  @param stSensor = CO2 Sensor Location Structure
   *  @param stGrid   = TRS Land Grid List Structure
   */

  public plotXYZData( int iPlot, plotXYStruct  stPlot,
                      iqstratHeadersListStruct stWells,
                      quakeSensorsStruct       stSensor,
                      utilLandGridListStruct   stGrid )
  {
    setBackground(Color.white);

    this.iPlot    = iPlot;
    this.stPlot   = stPlot;
    this.stWells  = stWells;
    this.stSensor = stSensor;
    this.stGrid   = stGrid;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    if (stPlot.iWo != 0)
      iWo = stPlot.iWo;

    if (stPlot.iHo != 0)
      iHo = stPlot.iHo;

    iXo = iXBegin;
    iYo = iHeight - iHo - iYBegin - iYBase;

    iX1 = iWidth  - iWo - iXBegin + iXMove;
//    iX1 = iWidth  - iWo - iXBegin;
    iY1 = iYBegin + iYMove;

    iRows = stPlot.iRows;

    if (stPlot.stSymbols != null)
      this.stSymbol = stPlot.stSymbols;

    this.iLegend = stPlot.iLegend;
    if (stPlot.stLegend != null)
      this.stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();

    if (stGrid != null)
      pGrid = new utilPlotLandGrid( stGrid,
                                    iXBegin,
                                    iYBegin,
                                    iWidth  + iXBegin,
                                    iHeight + iYBegin,
                                    stPlot.dXMinimum, stPlot.dXMaximum,
                                    stPlot.dYMinimum, stPlot.dYMaximum,
                                    iGrid );
  }

  /* =============================================================== *
   * ----------------------- DELETE METHODS ------------------------ *
   * =============================================================== */

  /** Method close()
   * <p> This method will set strings and arrays to null so Java can
   *     automatically clear memory
   */

  public void close()
  {
    stPlot   = null;
    stSensor = null;
    stGrid   = null;

    // Data

    iXp      = null; // Array of X Data converted to Pixels for 2D Start
    iYp      = null; // Array of X Data converted to Pixels for 2D Start
    iX       = null; // Array of Y Data converted to Pixels
    iY       = null; // Array of Z Data converted to Pixels
    stSymbol = null; // Array of Symbol and Color for Point
    stLegend = null; // Array of Symbol and Color for Legend

    if (pSymbol != null)
      pSymbol.close();
    pSymbol  = null;

    // Wells

    iWXp     = null; // Array of X Data converted to Pixels for 2D Start
    iWYp     = null; // Array of X Data converted to Pixels for 2D Start
    iWTD     = null;
    iWX      = null;
    iWY      = null;
    iWSymbol = null;

    // Sensors

    iSXp     = null; // Array of X Data converted to Pixels for 2D Start
    iSYp     = null; // Array of X Data converted to Pixels for 2D Start
    iSX      = null;
    iSY      = null;

    if (plotMap != null)
      plotMap.close();
    plotMap = null;
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setData(plotXYStruct stPlot)
   * <p> This method will change the Plot XY Data Structure
   * @param stPlot = the Plot XY Data Structure
   */

  public void setData(plotXYStruct stPlot)
  {
    this.stPlot = stPlot;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    if (stPlot.iWo != 0)
      iWo = stPlot.iWo;

    if (stPlot.iHo != 0)
      iHo = stPlot.iHo;

    iXo = iXBegin;
    iYo = iHeight - iHo - iYBegin - iYBase;

    iX1 = iWidth  - iWo - iXBegin + iXMove;
    iY1 = iYBegin + iYMove;

    iRows = stPlot.iRows;

    stSymbol = null;
    if (stPlot.stSymbols != null)
      stSymbol = stPlot.stSymbols;

    iLegend  = stPlot.iLegend;
    stLegend = null;
    if (stPlot.stLegend != null)
      stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();

    if (pGrid != null)
      pGrid.setXYLimt(stPlot.dXMinimum, stPlot.dXMaximum,
                      stPlot.dYMinimum, stPlot.dYMaximum );

    repaint();
  }

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    this.iLegend  = iLegend;
    this.stLegend = stLegend;
  }

  /** Method setLegendFlag()
   * <p> This method will set the Legend on or off by toggling the iLegend
   *     value
   * @param iLegend = The total number of legend values
   */

  public void setLegendFlag(int iLegend)
  {
    this.iLegend = iLegend;
    repaint();
  }

  /** Method setFluid()
   *  <p> This method will set the Fluid type and recompute the marker minerals
   */

  public void setFluid( int iFluid )
  {
  }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
    this.stPlot.sTitle1 = new String(sText);
    repaint();
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
    this.stPlot.sTitle2 = new String(sText);
    repaint();
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
    this.stPlot.sTitle3 = new String(sText);
    repaint();
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sXAxis      = new String(sText);
    this.stPlot.iXLog       = iGrid;
    this.stPlot.dXMaximum   = dMax;
    this.stPlot.dXMinimum   = dMin;
    this.stPlot.dXIncrement = dInc;
    this.stPlot.iXCycles    = iCycle;

    convertDataToPixels();

    if (pGrid != null)
      pGrid.setXYLimt(stPlot.dXMinimum, stPlot.dXMaximum,
                      stPlot.dYMinimum, stPlot.dYMaximum );

    repaint();
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sYAxis      = new String(sText);
    this.stPlot.iYLog       = iGrid;
    this.stPlot.dYMaximum   = dMax;
    this.stPlot.dYMinimum   = dMin;
    this.stPlot.dYIncrement = dInc;
    this.stPlot.iYCycles    = iCycle;


    convertDataToPixels();

    if (pGrid != null)
      pGrid.setXYLimt(stPlot.dXMinimum, stPlot.dXMaximum,
                      stPlot.dYMinimum, stPlot.dYMaximum );
    repaint();
  }

  /** Method setZAxis()
   * <p> This method will reset the Z-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setZAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sZAxis      = new String(sText);
    this.stPlot.iZLog       = iGrid;
    this.stPlot.dZMaximum   = dMax;
    this.stPlot.dZMinimum   = dMin;
    this.stPlot.dZIncrement = dInc;
    this.stPlot.iZCycles    = iCycle;


    convertDataToPixels();
    repaint();
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag)
  {
    this.bFlag = bFlag;
    repaint();
  }

  /** Method setTriangle()
   * <p> This method will set the triangle data
   * @param iData      = The total number points
   * @param sLithology = The lithology values
   */

  public void setTriangle(int iData, String sLithology[])
  {
  }

  /** Method setStatus()
   *  <p> Turn the Wells on or off.
   *  @param iaction = 0 wells off, 1 wells on.
   */

  public void setStatus(int iAction)
  {
    this.iAction = iAction;
    repaint();
  }

  /** Method setGrid()
   *  <p> Turn the TRS Grid on or off.
   *  @param iTRS = 0 wells off, 1 wells on.
   */

  public void setGrid(int iTRS)
  {
	this.iTRS = iTRS;
	repaint();
  }

  /* ================================================================== *
   * ------------------------ SUPPORT METHODS ------------------------- *
   * ================================================================== */

  /** Method void convertDataToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertDataToPixels()
  {
    int    i      = 0;
    int    iRatio = 0;
    double dXMin  = 0;
    double dXMax  = 0;
    double dYMin  = 0;
    double dYMax  = 0;
    double dZMin  = 0;
    double dZMax  = 0;

    iRows = stPlot.iRows;
    iX   = new int [iRows];
    iY   = new int [iRows];
    iXp  = new int [iRows];
    iYp  = new int [iRows];

    // Set the Y-Axis Data

    // IF the Y-Axis Data is LINEAR THEN
    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
    {
      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)(iX1-iXo) *
                       (stPlot.dY[i] - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iXp[i] = iXo + iRatio;

        iRatio = (int)((double)(iY1-iYo) *
                       (stPlot.dY[i] - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iYp[i] = iYo + iRatio;
      }
    }
    // IF the Y-Axis Data is LOG THEN
    else
    {
      dYMin = Math.log(stPlot.dYMinimum) / Math.log(10.0);
      dYMax = Math.log(stPlot.dYMaximum) / Math.log(10.0);

      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)(iX1-iXo) *
                       ((Math.log(stPlot.dY[i])/Math.log(10.0))-dYMin) /
                       (dYMax - dYMin));
        iXp[i] = iXo + iRatio;

        iRatio = (int)((double)(iY1-iYo) *
                       ((Math.log(stPlot.dY[i])/Math.log(10.0))-dYMin) /
                       (dYMax - dYMin));
        iYp[i] = iYo + iRatio;
      }
    }

    // Set the Z-Axis Data - This variable is plotted to the Y Direction

    // IF the Z-Axis Data is LINEAR THEN
    if (stPlot.iZLog == plot.plotXYStruct.LINEAR)
    {
      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iHo *
                       (stPlot.dZ[i] - stPlot.dZMinimum) /
                       (stPlot.dZMaximum - stPlot.dZMinimum));
        iY[i] = iHo + iYp[i] - iRatio;
      }
    }
    // IF the Z-Axis Data is LOG THEN
    else
    {
      dZMin = Math.log(stPlot.dZMinimum) / Math.log(10.0);
      dZMax = Math.log(stPlot.dZMaximum) / Math.log(10.0);

      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iHo *
                       ((Math.log(stPlot.dZ[i])/Math.log(10.0))-dZMin) /
                       (dZMax - dZMin));
        iY[i] = iHo + iYp[i] - iRatio;
      }
    }

    // Set the X-Axis Data - This variable is plotted to the X Direction

    // IF the X-Axis Data is LINEAR THEN
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
    {
      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iWo *
                       (stPlot.dX[i] - stPlot.dXMinimum) /
                       (stPlot.dXMaximum - stPlot.dXMinimum));
        iX[i] = iXp[i] + iRatio;
      }
    }
    // IF the X-Axis Data is LOG THEN
    else
    {
      dXMin = Math.log(stPlot.dXMinimum) / Math.log(10.0);
      dXMax = Math.log(stPlot.dXMaximum) / Math.log(10.0);

      for (i=0; i<iRows; i++)
      {
        iRatio = (int)((double)iWo *
                       ((Math.log(stPlot.dX[i])/Math.log(10.0))-dXMin) /
                       (dXMax - dXMin));
        iX[i] = iXp[i] + iRatio;
      }
    }

    convertWellDataToPixels();
    convertSensorDataToPixels();
    setOrder();
  }

  /** Method void convertWellDataToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertWellDataToPixels()
  {
    int    iRatio     = 0;
    int    i          = 0;
    double dkm        = 0.0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;

    if (stWells != null)
    {
      iWRows   = stWells.iCount;
      iWX      = new int[iWRows];
      iWY      = new int[iWRows];
      iWSymbol = new int[iWRows];
      iWXp     = new int[iWRows];
      iWYp     = new int[iWRows];
      iWTD     = new int[iWRows];

      for (i=0; i<iWRows; i++)
      {
        dLatitude = stWells.stItem[i].dLatitude;

        iRatio = (int)((double)(iX1-iXo) *
                       (dLatitude - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iWXp[i] = iXo + iRatio;

        iRatio = (int)((double)(iY1-iYo) *
                       (dLatitude - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iWYp[i] = iYo + iRatio;
      }

      for (i=0; i<iWRows; i++)
      {
		dkm = -0.0003048 * stWells.stItem[i].depth;

        if (dkm > stPlot.dZMaximum)
        {
		  dkm = stPlot.dZMaximum;
	    }

        iRatio = (int)((double)iHo *
                       (dkm - stPlot.dZMinimum) /
                       (stPlot.dZMaximum - stPlot.dZMinimum));

        iWTD[i] = iHo + iWYp[i] - iRatio;

        iRatio  = 0;
        iWY[i]  = iHo + iWYp[i] - iRatio;
      }


      for (i=0; i<iWRows; i++)
      {
        dLongitude = stWells.stItem[i].dLongitude;

        iRatio = (int)((double)iWo *
                       (dLongitude - stPlot.dXMinimum) /
                       (stPlot.dXMaximum - stPlot.dXMinimum));
        iWX[i] = iWXp[i] + iRatio;
      }

	  getWellSymbol();
    }
  }

  /** Method getWellSymbol()
   *  <p> This method will convert the Well Symbol to one of the recognized
   *      symbols.
   */

  private void getWellSymbol()
  {
    for (int i=0; i<iWRows; i++)
    {
      iWSymbol[i] = plot.plotWellStatusStruct.NONE;

      for (int j=0; j<stSymbols.iCount; j++)
      {
        if (stWells.stItem[i].status.equals(stSymbols.status[j][0]))
          iWSymbol[i] = stSymbols.iStatus[j];
      }
    }
  }

  /** Method void convertSensorDataToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertSensorDataToPixels()
  {
    int    iRatio     = 0;
    int    i          = 0;
    double dkm        = 0.0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;

    if (stSensor != null)
    {
      iSRows   = stSensor.iTotal;
      iSX      = new int[iSRows];
      iSY      = new int[iSRows];
      iSXp     = new int[iSRows];
      iSYp     = new int[iSRows];

      for (i=0; i<iSRows; i++)
      {
        dLatitude = stSensor.dLat[i];

        iRatio = (int)((double)(iX1-iXo) *
                       (dLatitude - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iSXp[i] = iXo + iRatio;

        iRatio = (int)((double)(iY1-iYo) *
                       (dLatitude - stPlot.dYMinimum) /
                       (stPlot.dYMaximum - stPlot.dYMinimum));
        iSYp[i] = iYo + iRatio;
      }

      for (i=0; i<iSRows; i++)
      {
        iRatio  = 0;
        iSY[i]  = iHo + iSYp[i] - iRatio;
      }


      for (i=0; i<iSRows; i++)
      {
        dLongitude = stSensor.dLong[i];

        iRatio = (int)((double)iWo *
                       (dLongitude - stPlot.dXMinimum) /
                       (stPlot.dXMaximum - stPlot.dXMinimum));
        iSX[i] = iSXp[i] + iRatio;
      }
    }
  }

  /** Method setOrder()
   *  <p> This method will set the order for plotting all the elements
   *
   *  The variable ipData is a double array [ # in data set ][ Well or Data Indeicator ]
   */

  private void setOrder()
  {
	int    i,j, m,n;
	double dXmin  = 0.0;
	double dYmin  = 0.0;
	double dXincr = 0.0;
	double dXmax  = 0.0;
	double dYmax  = 0.0;
	double dYincr = 0.0;
	int    ixdel  = 0;
	int    iydel  = 0;
	int    icnt   = 0;

    if (ipData != null)
    {
	  iTotal = 0;
	  ipData = null;
	}

	if ((stPlot != null) && (stWells != null))
	{
	  iTotal = stWells.iCount + stPlot.iRows;

	  ipData = new int[iTotal][2];

	  for (i=0; i<iTotal; i++)
	  {
 	    ipData[i][0] = -1;
 	    ipData[i][1] = -1;
	  }

  	  ixdel = (int) (( stPlot.dXMaximum - stPlot.dXMinimum ) / stPlot.dXIncrement );
  	  iydel = (int) Math.ceil(( stPlot.dYMaximum - stPlot.dYMinimum ) / stPlot.dYIncrement );

	  for (i=0; i<iydel; i++)
	  {
		dYmax = stPlot.dYMaximum - i * stPlot.dYIncrement;
		dYmin = stPlot.dYMaximum - (i+1) * stPlot.dYIncrement;
//System.out.println(i+" "+dYmin+" "+dYmax);

		for (j=0; j<ixdel; j++)
		{
		  dXmin = stPlot.dXMinimum + j * stPlot.dXIncrement;
		  dXmax = stPlot.dXMinimum + (j+1) * stPlot.dXIncrement;

		  for (m=0; m<stWells.iCount; m++)
		  {
			if ((stWells.stItem[m].dLatitude  >= dYmin) &&
			    (stWells.stItem[m].dLatitude  <  dYmax) &&
			    (stWells.stItem[m].dLongitude >= dXmin) &&
			    (stWells.stItem[m].dLongitude <  dXmax))
			{
			  if (icnt < iTotal)
			  {
				ipData[icnt][0] = m;
				ipData[icnt][1] = _WELL;
				icnt++;
			  }
			}
		  }

		  for (n=0; n<stPlot.iRows; n++)
		  {
			if ((stPlot.dY[n] >= dYmin) && (stPlot.dY[n] <  dYmax) &&
			    (stPlot.dX[n] >= dXmin) && (stPlot.dX[n] <  dXmax))
			{
			  if (icnt < iTotal)
			  {
				ipData[icnt][0] = n;
				ipData[icnt][1] = _DATA;
				icnt++;
			  }
			}
		  }
		}
	  }
	}
/*
	for (i=0; i<iTotal; i++)
	{
	  if (ipData[i][1] == _WELL)
        System.out.println("WELL :"+  ipData[i][0] +" "+
                           stWells.stItem[ipData[i][0]].dLatitude+" "+
                           stWells.stItem[ipData[i][0]].dLongitude);
      else if (ipData[i][1] == _DATA)
        System.out.println("DATA: "+ ipData[i][0] +" "+
                           stPlot.dY[ipData[i][0]]+" "+
                           stPlot.dX[ipData[i][0]]);
	}
*/
  }

  /* ================================================================== *
   * -------------------------- DRAW METHODS -------------------------- *
   * ================================================================== */

  /** Method drawTitles()
   *  <p> This method will draw the titles of the Plot.
   *  @param g - Graphics pointer.
   */

  public void drawTitles(Graphics g)
  {
    int iTemp = 0;

    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 14) );

    if (stPlot.sTitle1 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle1.length())));
      g.drawString(stPlot.sTitle1, iTemp / 2, 10);
    }

    if (stPlot.sTitle2 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle2.length())));
      g.drawString(stPlot.sTitle2, iTemp / 2, 30);
    }

    if (stPlot.sTitle3 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle3.length())));
      g.drawString(stPlot.sTitle3, iTemp / 2, 50);
    }
  }

  /** Method drawLabels()
   *  <p> This method will draw the Linear Plot Labels.
   *  @param g - Graphics pointer.
   */

  public void drawLabels(Graphics g)
  {
    int ivalue;
    Graphics2D g2 = (Graphics2D) g;
    double dRatio = 1;

    g.setFont( new Font("Monospaced", Font.BOLD, 11) );
    g.setColor(Color.black);

    if (stPlot.sXAxis != null)
    {
      ivalue = iXo + iWo/2 - 3*(stPlot.sXAxis).length();
      g.drawString(stPlot.sXAxis, iXo+10, iYo+iHo+35);
    }

    if (stPlot.sYAxis != null)
    {
      ivalue = iYo+iHo;

      dRatio = (iYo-iY1) / (iX1-iXo);
      g2.translate(iXo+iWo+45, iYo+iHo+20);
      g2.rotate( -1.0*Math.atan(dRatio));

      g2.drawString(stPlot.sYAxis, 0, 0);

      g2.rotate( Math.atan(dRatio));
      g2.translate(-1.0*(iXo+iWo+45), -1.0*(iYo+iHo+20));
	}

    if (stPlot.sZAxis != null)
    {
      ivalue = iY1 + iHo + - 2*(stPlot.sZAxis).length();

      g2.translate(iWidth-20+iXMove, ivalue);
      g2.rotate( -1.0*(Math.PI / 2.0));
      g2.drawString(stPlot.sZAxis, 0, 0);
      g2.rotate( (Math.PI / 2.0));
      g2.translate(-1.0*(iWidth-20+iXMove), -1.0*ivalue);
    }
  }

  /** Method drawGrid()
   *  <p> This method will draw the Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
	drawGridXZ(g);
	drawGridYZ(g);
	drawGridXY(g);
  }

  /** Method drawGridXZ()
   *  <p> This method will draw the XZ Grid
   *  @param g - Graphics pointer.
   */

  public void drawGridXZ(Graphics g)
  {
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
    {
      drawGridXZ_LinearX(g);
	}
	else
	{
      drawGridXZ_LogX(g);
	}

    if (stPlot.iZLog == plot.plotXYStruct.LINEAR)
    {
      drawGridXZ_LinearZ(g);
    }
    else
    {
      drawGridXZ_LogZ(g);
    }
  }

  /** Method drawGridXZ_LinearX()
   *  <p> This method will draw Grid XZ the Linear X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXZ_LinearX(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    double dx     = 0.0;
    double del    = 0.0;

    iXinc = (int) ((double)iWo * stPlot.dXIncrement /
                   (stPlot.dXMaximum - stPlot.dXMinimum));

    idraw = iX1;
    while (idraw < iX1+iWo+5)
    {
      dx     = stPlot.dXMinimum + (double) i * stPlot.dXIncrement;
      iRatio = (int) ((double) iWo *
                      (dx - stPlot.dXMinimum) /
                      (stPlot.dXMaximum - stPlot.dXMinimum));
      idraw = iX1 + iRatio;

      if (idraw < iX1+iWo+5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(idraw, iY1, idraw, iY1+iHo);

        g.setColor(Color.black);
/*
        for (j=1; j<5; j++)
        {
          idel = idraw + (j * iXinc) / 5;
          if (idel < iX1+iWo)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idel, iY1, idel, iY1+iHo);
          }
        }
*/
      }

      i++;
    }
  }

  /** Method drawGridXZ_LogX()
   *  <p> This method will draw Grid XZ the Log X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXZ_LogX(Graphics g)
  {
    int    i, j;
    int    l=1;
    int    k=0;
    int    iXinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;

    iXinc = 1 + iWo / stPlot.iXCycles;

    g.setColor(Color.darkGray);
    g.drawLine(iX1, iY1, iX1, iY1+iHo);

    for (i=iX1+iXinc; i<iX1+iWo+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);
      g.drawLine(i, iY1, i, iY1+iHo);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iXinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - iXinc + (int) dTemp;

        g.drawLine(jXinc, iY1, jXinc, iY1+iHo);
      }
    }

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    del = stPlot.dXIncrement;
    for (i=iX1+iXinc; i<iX1+iWo+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);

      dLabel = (double) k * 10.0 * stPlot.dXMinimum;
      k = k * 10;

      g.setColor(Color.black);
      del = stPlot.dXIncrement + del;
    }
  }

  /** Method drawGridXZ_LinearZ()
   *  <p> This method will draw Grid XZ the Linear Z-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXZ_LinearZ(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iYinc  = 0;
    double dy     = 0.0;
    double del    = 0.0;

    iYinc = (int) ((double)iHo * stPlot.dZIncrement /
                   (stPlot.dZMaximum - stPlot.dZMinimum));

    idraw = iY1 + iHo;
    while (idraw > iY1-5)
    {
      dy = stPlot.dZMinimum + (double) i * stPlot.dZIncrement;
      iRatio = (int)((double)iHo *
                     (dy - stPlot.dZMinimum) /
                     (stPlot.dZMaximum - stPlot.dZMinimum));
      idraw = iHo + iY1 - iRatio;

      if (idraw > iY1-5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(iX1, idraw, iX1+iWo, idraw);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );
        dy = Math.round(dy*1000)/1000.0;
        g.drawString("" + dy, iX1+iWo+5, idraw);
/*
        for (j=1; j<5; j++)
        {
          idel = idraw - (j * iYinc) / 5;
          if (idel > iY1)
          {
            g.setColor(Color.lightGray);
            g.drawLine(iX1, idel, iX1+iWo, idel);
          }
        }
*/
      }

      i++;
    }
  }

  /** Method drawGridXZ_LogZ()
   *  <p> This method will draw Grid XZ the Log Z-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXZ_LogZ(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    k = 1;
    int    iYinc  = 0;
    int    jXinc  = 0;
    double dy     = 0.0;
    double dTemp  = 0.0;
    double dXinc  = 0.0;

    iYinc = 1 + iHo / stPlot.iZCycles;

    for (i=iY1+iHo; i>iY1; i-=iYinc)
    {
      k = i;
      g.setColor(Color.darkGray);
      g.drawLine(iX1, i, iX1+iWo, i);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iYinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - (int) dTemp;

        g.drawLine(iX1, jXinc, iX1+iWo, jXinc);
      }
    }

    g.setColor(Color.darkGray);
    g.drawLine(iX1, k-iYinc, iX1+iWo, k-iYinc);

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    g.drawString("" + stPlot.dZMinimum, iX1+iWo+5, iY1+iHo);

    for (i=iY1+iHo; i>iY1; i-=iYinc)
    {
      dy = (double) k * 10.0 * stPlot.dZMinimum;
      k = k * 10;
      g.drawString("" + dy, iX1+iWo+5, i-iYinc);
    }
  }

  /** Method drawGridYZ()
   *  <p> This method will draw the YZ Grid
   *  @param g - Graphics pointer.
   */

  public void drawGridYZ(Graphics g)
  {
    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
    {
      drawGridYZ_LinearY(g);
    }
    else
    {
	  drawGridYZ_LogY(g);
	}

    if (stPlot.iZLog == plot.plotXYStruct.LINEAR)
    {
      drawGridYZ_LinearZ(g);
    }
    else
    {
      drawGridYZ_LogZ(g);
    }
  }

  /** Method drawGridYZ_LinearY()
   *  <p> This method will draw Grid YZ the Linear Y-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridYZ_LinearY(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    int    iYinc  = 0;
    double dx     = 0.0;
    double dx1    = 0.0;
    double del    = 0.0;
    int    idelx  = 0;
    int    idely  = 0;
    int    iW1    = 0;
    int    iXp    = 0;
    int    iYp    = 0;

    iW1   = (int) Math.sqrt((double)((iX1-iXo) * (iX1-iXo)) + (double)((iY1-iYo) * (iY1-iYo)));

    iXinc = (int) ((double)iW1 * stPlot.dYIncrement /
                   (stPlot.dYMaximum - stPlot.dYMinimum));

    idraw = iXo;
    while (iXp < iX1)
    {
      dx     = stPlot.dYMinimum + (double) i * stPlot.dYIncrement;

      iRatio = (int)((double)(iX1-iXo) *
                     (dx - stPlot.dYMinimum) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
      iXp = iXo + iRatio;

      iRatio = (int)((double)(iY1-iYo) *
                     (dx - stPlot.dYMinimum) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
      iYp = iYo + iRatio;

      if (iXp < iX1)
      {
        g.setColor(Color.darkGray);
        g.drawLine(iXp, iYp, iXp, iYp+iHo);

        dx1 = stPlot.dYIncrement/5.0;
/*
        for (j=1; j<5; j++)
        {
          iRatio = (int)((double)(iX1-iXo) * (j*dx1) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
          idelx = iXp + iRatio;

          iRatio = (int)((double)(iY1-iYo) * (j*dx1) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
          idely = iYp + iRatio;

          if (idelx < iX1)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idelx, idely, idelx, idely+iHo);
          }
        }
*/
      }

      i++;
    }
  }

  /** Method drawGridYZ_LogY()
   *  <p> This method will draw Grid YZ the Log Y-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridYZ_LogY(Graphics g)
  {
    int    i, j;
    int    l=1;
    int    k=0;
    int    iXinc;
    int    Xinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;
    int    iW1    = 0;
    int    iXp    = 0;
    int    iYp    = 0;
    int    iXp1   = 0;
    int    iYp1   = 0;
    int    iTemp  = 0;

    iW1   = (int) Math.sqrt((double)((iX1-iXo) * (iX1-iXo)) + (double)((iY1-iYo) * (iY1-iYo)));
    iXinc = iW1 / stPlot.iYCycles;

    g.setColor(Color.darkGray);
    g.drawLine(iXo, iYo, iXo, iYo+iHo);

    for (i=0; i<stPlot.iYCycles+1; i++)
    {
      iXp = iXo + i * (iX1 - iXo) / stPlot.iYCycles;
      iYp = iYo + i * (iY1 - iYo) / stPlot.iYCycles;

      g.setColor(Color.darkGray);
      g.drawLine(iXp, iYp, iXp, iYp+iHo);

      g.setColor(Color.lightGray);
      if (iXp < iX1)
      {
        for (j=2; j<10; j++)
        {
          dXinc = (double) j;
          iTemp = (int) (1.0 + (double) iXinc * Math.log(j) / Math.log(10.0));

          iXp1 = iXp + iTemp * (iX1 - iXo) / iW1;
          iYp1 = iYp + iTemp * (iY1 - iYo) / iW1;

          g.drawLine(iXp1, iYp1, iXp1, iYp1+iHo);
        }
      }
    }
  }

  /** Method drawGridYZ_LinearZ()
   *  <p> This method will draw Grid YZ the Linear Z-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridYZ_LinearZ(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    idraw  = 0;
    int    idel1  = 0;
    int    idel2  = 0;
    int    iRatio = 0;
    int    iYinc  = 0;
    double dy     = 0.0;
    double del    = 0.0;
    int    iYp1   = 0;
    int    iYp2   = 0;

    iYinc = (int) ((double)iHo * stPlot.dZIncrement /
                   (stPlot.dZMaximum - stPlot.dZMinimum));

    iYp1 = iYo + iHo;
    while (iYp1 > iYo-5)
    {
      dy = stPlot.dZMinimum + (double) i * stPlot.dZIncrement;
      iRatio = (int)((double)iHo *
                     (dy - stPlot.dZMinimum) /
                     (stPlot.dZMaximum - stPlot.dZMinimum));
      iYp1 = iHo + iYo - iRatio;
      iYp2 = iHo + iY1 - iRatio;

      if (iYp1 > iYo-5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(iXo, iYp1, iX1, iYp2);
/*
        for (j=1; j<5; j++)
        {
          idel1 = iYp1 - (j * iYinc) / 5;
          idel2 = iYp2 - (j * iYinc) / 5;
          if (idel1 > iYo)
          {
            g.setColor(Color.lightGray);
            g.drawLine(iXo, idel1, iX1, idel2);
          }
        }
*/
      }

      i++;
    }
  }

  /** Method drawGridYZ_LogZ()
   *  <p> This method will draw Grid YZ the Log Z-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridYZ_LogZ(Graphics g)
  {
    int    i = 0;
    int    j = 0;
    int    k = 1;
    int    iYinc  = 0;
    int    jXinc  = 0;
    double dy     = 0.0;
    double dTemp  = 0.0;
    double dXinc  = 0.0;
    int    iYp1   = 0;
    int    iYp2   = 0;

    iYinc = 1 + iHo / stPlot.iZCycles;

    for (i=iYo+iHo; i>iYo; i-=iYinc)
    {
      k = i;
      g.setColor(Color.darkGray);
      g.drawLine(iXo, i, iX1, i+iY1-iYo);

      g.setColor(Color.lightGray);

      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iYinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - (int) dTemp;

        g.drawLine(iXo, jXinc, iX1, jXinc+iY1-iYo);
      }
    }

    g.setColor(Color.darkGray);
    g.drawLine(iXo, k-iYinc, iX1, k-iYinc+iY1-iYo);
  }

  /** Method drawGridXY()
   *  <p> This method will draw the XY Grid
   *  @param g - Graphics pointer.
   */

  public void drawGridXY(Graphics g)
  {
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
    {
      drawGridXY_LinearX(g);
    }
    else
    {
      drawGridXY_LogX(g);
	}

    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
    {
      drawGridXY_LinearY(g);
    }
    else
    {
	  drawGridXY_LogY(g);
	}
  }

  /** Method drawGridXY_LinearY()
   *  <p> This method will draw Grid XY the Linear Y-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXY_LinearY(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idelx  = 0;
    int    idely  = 0;
    int    iRatio = 0;
    double dx     = 0.0;
    double dx1    = 0.0;
    int    iXp    = 0;
    int    iYp    = 0;

    while (iXp < iX1)
    {
      dx     = stPlot.dYMinimum + (double) i * stPlot.dYIncrement;

      iRatio = (int)((double)(iX1-iXo) *
                     (dx - stPlot.dYMinimum) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
      iXp = iXo + iRatio;

      iRatio = (int)((double)(iY1-iYo) *
                     (dx - stPlot.dYMinimum) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
      iYp = iYo + iRatio;

      if (iXp < iX1)
      {
        g.setColor(Color.darkGray);
        g.drawLine(iXp, iYp+iHo, iXp+iWo, iYp+iHo);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );
        dx = Math.round(dx*1000)/1000.0;
        g.drawString(""+dx, iXp+iWo+5, iYp+iHo+15);

        dx1 = stPlot.dYIncrement/5.0;

        for (j=1; j<5; j++)
        {
          iRatio = (int)((double)(iX1-iXo) * (j*dx1) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
          idelx = iXp + iRatio;

          iRatio = (int)((double)(iY1-iYo) * (j*dx1) /
                     (stPlot.dYMaximum - stPlot.dYMinimum));
          idely = iYp + iRatio;

          if (idelx < iX1)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idelx, idely+iHo, idelx+iWo, idely+iHo);
          }
        }
      }

      i++;
    }
  }

  /** Method drawGridXY_LogY()
   *  <p> This method will draw Grid YZ the Log Y-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXY_LogY(Graphics g)
  {
    int    i, j;
    int    l=1;
    int    k=1;
    int    iXinc;
    int    Xinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;
    int    iW1    = 0;
    int    iXp    = 0;
    int    iYp    = 0;
    int    iXp1   = 0;
    int    iYp1   = 0;
    int    iTemp  = 0;

    iW1   = (int) Math.sqrt((double)((iX1-iXo) * (iX1-iXo)) + (double)((iY1-iYo) * (iY1-iYo)));
    iXinc = iW1 / stPlot.iYCycles;

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.darkGray);
    g.drawLine(iXo, iYo, iXo, iYo+iHo);

    for (i=0; i<stPlot.iYCycles+1; i++)
    {
      iXp = iXo + i * (iX1 - iXo) / stPlot.iYCycles;
      iYp = iYo + iHo + i * (iY1 - iYo) / stPlot.iYCycles;

      g.setColor(Color.darkGray);
      g.drawLine(iXp, iYp, iXp+iWo, iYp);

      g.setColor(Color.black);
      dLabel = (double) k * stPlot.dYMinimum;
      g.drawString("" + dLabel, iXp+iWo+5, iYp+15);
      k = k * 10;

      g.setColor(Color.lightGray);
      if (iXp < iX1)
      {
        for (j=2; j<10; j++)
        {
          dXinc = (double) j;
          iTemp = (int) (1.0 + (double) iXinc * Math.log(j) / Math.log(10.0));

          iXp1 = iXp + iTemp * (iX1 - iXo) / iW1;
          iYp1 = iYp + iTemp * (iY1 - iYo) / iW1;

          g.drawLine(iXp1, iYp1, iXp1+iWo, iYp1);
        }
      }
    }
  }

  /** Method drawGridXY_LinearX()
   *  <p> This method will draw Grid XY the Linear X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXY_LinearX(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    double dx     = 0.0;
    double del    = 0.0;

    iXinc = (int) ((double)iWo * stPlot.dXIncrement /
                   (stPlot.dXMaximum - stPlot.dXMinimum));

    idraw = iX1;
    while (idraw < iXo+iWo+5)
    {
      dx     = stPlot.dXMinimum + (double) i * stPlot.dXIncrement;
      iRatio = (int) ((double) iWo *
                      (dx - stPlot.dXMinimum) /
                      (stPlot.dXMaximum - stPlot.dXMinimum));
      idraw = iXo + iRatio;

      if (idraw < iXo+iWo+5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(idraw, iYo+iHo, idraw-iXo+iX1, iY1+iHo);


        if (idraw < iXo+iWo)
        {
          g.setColor(Color.black);
          g.setFont( new Font("Monospaced", Font.BOLD, 10) );
          dx = Math.round(dx*1000)/1000.0;
          g.drawString(""+dx, idraw-10, iYo+iHo+15);
	    }

        for (j=1; j<5; j++)
        {
          idel = idraw + (j * iXinc) / 5;
          if (idel < iXo+iWo)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idel, iYo+iHo, idel-iXo+iX1, iY1+iHo);
          }
        }
      }

      i++;
    }
  }

  /** Method drawGridXY_LogX()
   *  <p> This method will draw Grid XY the Log X-Axis Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGridXY_LogX(Graphics g)
  {
    int    i, j;
    int    l=1;
    int    k=0;
    int    iXinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;

    iXinc = 1 + iWo / stPlot.iXCycles;

    g.setColor(Color.darkGray);
    g.drawLine(iXo, iYo+iHo, iX1, iY1+iHo);

    for (i=iXo+iXinc; i<iXo+iWo+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);
      g.drawLine(i, iYo+iHo, i+iX1-iXo, iY1+iHo);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iXinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - iXinc + (int) dTemp;

        g.drawLine(jXinc, iYo+iHo, jXinc+iX1-iXo, iY1+iHo);
      }
    }

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    del = stPlot.dXIncrement;
    g.drawString("" + stPlot.dXMinimum, iXo-10, iYo+iHo+15);

    for (i=iXo+iXinc; i<iXo+iWo; i+=iXinc)
    {
      g.setColor(Color.darkGray);

      dLabel = (double) k * 10.0 * stPlot.dXMinimum;
      k = k * 10;

      g.setColor(Color.black);
      g.drawString("" + dLabel, i-10, iYo+iHo+15);
      del = stPlot.dXIncrement + del;
    }
  }

  /** Method drawShadowData()
   * <p> This method will plot the data shadow on the bottom XY grid
   *  @param g - Graphics pointer.
   */

  public void drawShadowData(Graphics g)
  {
    int iDraw = 0;
    int iX1   = 0;
    int iX2   = 0;
    int iY1   = 0;
    int iY2   = 0;

    for (int i=0; i<iRows; i++)
    {
      iX1 = iX[i];
      iY1 = iY[i];

      if ( (iX1 > iXp[i]) && (iX1 < iXp[i] + iWo )  &&
           (iY1 > iYp[i]) && (iY1 < iYp[i] + iHo)  &&
           (stPlot.dY[i] >= stPlot.dYMinimum) &&
           (stPlot.dY[i] <= stPlot.dYMaximum))
      {
        iDraw = 1;

        if (iDraw == 1)
        {
          g.setColor(Color.lightGray);
          g.drawOval(iX[i]-4, iYp[i]+iHo, 8, 8);
        }
      }
    }
  }

  /** Method drawData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawData(Graphics g)
  {
	int io = -1;
	int id = 0;

	for (int i=0; i<iTotal; i++)
	{
	  if (ipData[i][1] == _WELL)
	  {
        io = ipData[i][0];

        if (iAction == _ON)
        {
          if ((iWX[io] >= iWXp[io]) && (iWX[io] <= iWXp[io] + iWo )   &&
              (iWY[io] >= iWYp[io]) && (iWY[io] <= iWYp[io] + iHo)    &&
              (stWells.stItem[io].dLatitude >= stPlot.dYMinimum) &&
              (stWells.stItem[io].dLatitude <= stPlot.dYMaximum))
          {
            plotMap.drawWellSymbol(g, iWSymbol[io], iWX[io], iWY[io]);
            g.setColor(new Color(220,220,255));
//            g.setColor(Color.lightGray);
            g.drawLine(iWX[io], iWY[io]-5, iWX[io], iWYp[io]);

            g.setColor(Color.red);
            g.drawLine(iWX[io], iWTD[io], iWX[io], iWYp[io]);
		  }
	    }
      }
      else if (ipData[i][1] == _DATA)
      {
        io = ipData[i][0];

        if ( (iX[io] > iXp[io]) && (iX[io] < iXp[io] + iWo )  &&
             (iY[io] > iYp[io]) && (iY[io] < iYp[io] + iHo)  &&
             (stPlot.dY[io] >= stPlot.dYMinimum) &&
             (stPlot.dY[io] <= stPlot.dYMaximum))
        {
          pSymbol.drawSymbol(g, stSymbol[io], iX[io], iY[io]);
		}
	  }
	}
  }

  /** Method drawPlotData()
   * <p> This method will plot the data
   *  @param g - Graphics pointer.
   */

  public void drawPlotData(Graphics g)
  {
    int iDraw = 0;
    int iX1   = 0;
    int iX2   = 0;
    int iY1   = 0;
    int iY2   = 0;

    for (int i=0; i<iRows; i++)
    {
      iX1 = iX[i];
      iY1 = iY[i];

      if ( (iX1 > iXp[i]) && (iX1 < iXp[i] + iWo )  &&
           (iY1 > iYp[i]) && (iY1 < iYp[i] + iHo)  &&
           (stPlot.dY[i] >= stPlot.dYMinimum) &&
           (stPlot.dY[i] <= stPlot.dYMaximum))
      {
        iDraw = 1;

        if (iDraw == 1)
        {
//          g.setColor(Color.lightGray);
//          g.drawOval(iX[i]-4, iYp[i]+iHo, 8, 8);

          pSymbol.drawSymbol(g, stSymbol[i], iX[i], iY[i]);
        }
      }
    }
  }

  /** Method drawLegend()
   *  <p> This method will create the Legend
   *  @param g        = Graphics pointer.
   */

  public void drawLegend(Graphics g)
  {
    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);

    if (stPlot.sLegTitle.length() > 0)
      g.drawString(stPlot.sLegTitle, iWidth+10, iY1-13);

    for (int i=0; i<iLegend; i++)
    {
      pSymbol.drawSymbol(g, stLegend[i], iWidth+10, iY1+i*13);

      g.setColor(Color.black);
      g.drawString(stLegend[i].symbol, iWidth+10+8, iY1+i*13);
    }

    g.setColor(Color.black);
    g.drawString("USGS", iWidth+10+8, iY1+iLegend*13+13);
    pSymbol.drawSymbol(g,
                       plot.plotSymbolStruct.FILLED,
                       plot.plotSymbolStruct.DIAMOND,
                       iWidth+10, iY1+iLegend*13+13);

    g.drawString("CO2 Sensor Array", iWidth+10+8, iY1+iLegend*13+26);
    pSymbol.drawSymbol(g,
                       plot.plotSymbolStruct.FILLED,
                       plot.plotSymbolStruct.TRIANGLE,
                       iWidth+10, iY1+iLegend*13+26);

    g.drawString("CO2 Sensor Array > 5 km", iWidth+10+8, iY1+iLegend*13+39);
    pSymbol.drawSymbol(g,
                       plot.plotSymbolStruct.EMPTY,
                       plot.plotSymbolStruct.TRIANGLE,
                       iWidth+10, iY1+iLegend*13+39);

    g.drawString("* Distances are measured",    iWidth+10, iY1+iLegend*13+52);
    g.drawString("with respect to the KGS 2-32", iWidth+10, iY1+iLegend*13+65);
    g.drawString("CO2-EOR injection well,",     iWidth+10, iY1+iLegend*13+78);
    g.drawString("located in 32-31S-1W.",       iWidth+10, iY1+iLegend*13+91);

    plotMap.drawWellSymbol(g, plot.plotWellStatusStruct.NONE, iWidth+10, iY1+iLegend*13+117);
    g.drawString("Wellington KGS Wells", iWidth+10+8, iY1+iLegend*13+120);

    plotMap.drawWellSymbol(g, plot.plotWellStatusStruct.SWD, iWidth+10, iY1+iLegend*13+133);
    g.drawString("Salt Water Disposal Wells", iWidth+10+8, iY1+iLegend*13+136);

    g.setColor(new Color(165, 42, 42));
    g.fillRect(iWidth+5, iY1+iLegend*13+144, 10, 10);
    g.setColor(Color.black);
    g.drawString("CO2 Seismic Sensors", iWidth+10+8, iY1+iLegend*13+151);

    g.setColor(Color.black);
    g.drawString("(*) Accuracy in depth and", iWidth+10, iY1+iHeight-180-52);
    g.drawString("location is significantly", iWidth+10, iY1+iHeight-180-39);
    g.drawString("decreased at distances",    iWidth+10, iY1+iHeight-180-26);
    g.drawString("greater than 5km from the", iWidth+10, iY1+iHeight-180-13);
    g.drawString("CO2 array.",                iWidth+10, iY1+iHeight-180);
  }

  /** Method drawWells()
   *  <p> This method will draw the Linear Plot Curves.
   *  @param g - Graphics pointer.
   */

  public void drawWells(Graphics g)
  {
    for (int i=0; i<iWRows; i++)
    {
      if ((iWX[i] >= iWXp[i]) && (iWX[i] <= iWXp[i] + iWo )   &&
          (iWY[i] >= iWYp[i]) && (iWY[i] <= iWYp[i] + iHo)    &&
          (stWells.stItem[i].dLatitude >= stPlot.dYMinimum) &&
          (stWells.stItem[i].dLatitude <= stPlot.dYMaximum))
      {
        plotMap.drawWellSymbol(g, iWSymbol[i], iWX[i], iWY[i]);
	  }
	}
  }

  /** Method drawSensors()
   *  <p> This method will draw the Linear Plot Curves.
   *  @param g - Graphics pointer.
   */

  public void drawSensors(Graphics g)
  {
    for (int i=0; i<iSRows; i++)
    {
      if ((iSX[i] >= iSXp[i]) && (iSX[i] <= iSXp[i] + iWo )   &&
          (iSY[i] >= iSYp[i]) && (iSY[i] <= iSYp[i] + iHo)    &&
          (stSensor.dLat[i] >= stPlot.dYMinimum) &&
          (stSensor.dLat[i] <= stPlot.dYMaximum))
      {
        g.setColor(new Color(165, 42, 42));
        g.fillRect(iSX[i]-5, iSY[i]-5, 10, 10);
	  }
	}
  }

  /** Method draw()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void draw(Graphics g)
  {
    drawTitles(g);
    if ((iTRS == _ON) && (pGrid != null))
      pGrid.draw3D(g, iXo, iYo, iX1, iY1, iHo, iWo);
    else
    {
      drawLabels(g);
      drawGrid(g);
	}

    g.setColor(Color.black);

    if (stPlot.sLegRange.length() > 0)
      g.drawString(stPlot.sLegRange, iWidth+10, iY1-26);

    if (stPlot.bLegend) drawLegend(g);

    drawSensors(g);
    drawShadowData(g);
    drawData(g);
//    drawWells(g);
//    drawPlotData(g);
  }

  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
	draw(g);
  }
}