/*
 * @quakeStartAppPanel.java Version 1.1 08/03/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnStruct;                       // Common Data Structure

import iqstrat.iqstratHeadersListStruct;    // Well Header List Structure
import iqstrat.io.ReadHeadersXMLFile;       // Read Well Header XML Stream

import quake.quakeDataListStruct;           // Quake Data List Structure
import quake.quakeSeisStruct;               // Quake Seismic Data Structure
import quake.io.ReadQuakeXMLFile;           // Read Quake XML Stream from KGS DB

import quake.gui.quakePlotTimeMagDataFrame; // Earthquake Mag vs Time Plot Frame
import quake.gui.quake3DPlotControlFrame;   // Earthquake Location 3D Plot Frame
import quake.gui.quakeAnimationFrame;       // Earthquake animation 3D Plot Frame

import quake.io.ReadSensorXMLFile;          // Read Sensor Location Data from DB
import quake.quakeSensorsStruct;            // Sensor Data Structure

import util.utilLandGridFile;               // Read TRS Land Grid File
import util.utilLandGridStruct;             // TRS Land Grid Data Structure
import util.utilLandGridListStruct;         // TRS Land Grid List Structure

/** Class quakeStartAppPanel
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 08/03/2015
 *  @author  John R. Victorine
 */

public class quakeStartAppPanel extends JPanel implements ActionListener
{
  public final static int _MAG_DATE = 0; // Create Magnitude vs Date Plot
  public final static int _3D_PLOT  = 1; // Create 3D Plot of Location
  public final static int _ANIMATE  = 2; // Create 3D Plot Animation

  // Input Variables

  private int        iType    = _MAG_DATE; // Type of Plot to create
  private JFrame     parent   = null;      // Frame calling this panel
  private cmnStruct  stStruct = null;      // Global Common Data Structure

  // Global Variable

  private quakePlotTimeMagDataFrame frame        = null; // Earthquake Mag vs Time Plot Frame
  private quake3DPlotControlFrame   frame3D      = null; // Earthquake 3D Plot Frame
  private quakeAnimationFrame       frame3DMovie = null; // Earthquake animation 3D Plot Frame

  private iqstratHeadersListStruct  stWells      = null; // headers Information Data Structure
  private quakeDataListStruct       stList       = null; // Quake Data List Structure
  private quakeSensorsStruct        stSensor     = null; // CO2 Seismic Sensor Locations Structure

  private utilLandGridListStruct    stLandGrid   = null; // Land Grid List Data

  // Global Panel Widgets

  private JButton    btn      = null;
  private JButton    btnExit  = new JButton();

  /** Constructor quakeStartAppPanel( JFrame parent, cmnStruct stStruct )
   *  <p> This is the Constructor for this class.
   *  @param parent   = the frame calling this panel
   *  @param stStruct = Global Common Data Structure
   */

  public quakeStartAppPanel( JFrame parent, cmnStruct stStruct )
  {
    try
    {
      this.parent   = parent;
      this.stStruct = stStruct;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** Constructor quakeStartAppPanel(
   *                  int iType, JFrame parent, cmnStruct stStruct )
   *  <p> This is the Constructor for this class.
   *  @param iType    = the type of plot to create
   *  @param parent   = the frame calling this panel
   *  @param stStruct = Global Common Data Structure
   */

  public quakeStartAppPanel( int iType, JFrame parent, cmnStruct stStruct )
  {
    try
    {
	  this.iType    = iType;
      this.parent   = parent;
      this.stStruct = stStruct;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlCenter = new JPanel();
    JPanel pnlBase   = new JPanel();

    JLabel lbl       = new JLabel();

    // Build Panel

    this.setLayout( new BorderLayout() );
    this.setBackground(Color.white);

    pnlCenter.setLayout( new BorderLayout() );
    pnlCenter.setBackground(Color.white);

    btn = new JButton( createImageIcon( cmn.cmnStruct.SEISMIC_APP) );
    btn.setBackground(Color.white);
    btn.addActionListener( this );

    lbl.setHorizontalAlignment(SwingConstants.CENTER);
    lbl.setFont(new java.awt.Font("Dialog", 1, 20));
    lbl.setBackground(Color.white);
    lbl.setText("Enter");

    pnlBase.setLayout( new GridLayout() );
    pnlBase.setBackground(Color.white);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.add( pnlCenter,    BorderLayout.CENTER );
    pnlCenter.add( btn,     BorderLayout.CENTER );
    pnlCenter.add( lbl,     BorderLayout.SOUTH );

    if (parent != null)
    {
      this.add( pnlBase,    BorderLayout.SOUTH );
      pnlBase.add( btnExit, null );
    }
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE METHOD ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stStruct = null;  // Global IQSTRAT Data Structure

    if (stWells != null)
      stWells.delete();
    stWells = null;

    if (stList != null)
      stList.delete();
    stList = null;

    if (stLandGrid != null)
      stLandGrid.delete();
    stLandGrid = null;

    if (stSensor != null)
      stSensor.delete();
    stSensor = null;

    if (frame != null)
      frame.close();
    frame    = null;

    if (frame3D != null)
      frame3D.close();
    frame3D    = null;

    if (frame3DMovie != null)
      frame3DMovie.close();
    frame3DMovie    = null;

    // Global Panel Widgets

    btn      = null;
    btnExit  = null;

    if (parent != null)
    {
      parent.dispose();
      parent = null;

      System.exit(0);
    }
  }

  /* ===================================================================== *
   * ------------------------ READ QUAKE DATA ---------------------------- *
   * ===================================================================== */

  /** Method readData()
   * <p> This method will read the Earthquake Data from the KGS DB xml stream
   */

  public void readData()
  {
  	ReadQuakeXMLFile pRead = new ReadQuakeXMLFile(quake.io.ReadQuakeXMLFile.URL);
    stList = pRead.Process( cmn.cmnStruct.EARTHQUAKES );
  }

  /** Method readArrayData()
   * <p> This method will read the Earthquake Data from the KGS DB xml stream
   */

  public void readArrayData()
  {
    quakeDataListStruct st = null;

  	ReadQuakeXMLFile pRead = new ReadQuakeXMLFile(quake.io.ReadQuakeXMLFile.URL);
    st = pRead.Process( cmn.cmnStruct.EVENTS );

    if ((stList != null) && (st != null))
    {
	  for (int i=0; i<st.iCount; i++)
	  {
		stList = quake.quakeDataUtility.add( st.stItem[i], stList );
	  }
	}
  }

  /** Method readWells()
   * <p> This method will read the User information xml data
   * @param  iMethod = The Method used in retrieving the XML
   * @param  sPath   = The url or File Path to use
   * @return stUser  = The User Data Structure
   */

  public void readWells()
  {
    ReadHeadersXMLFile pRead = new ReadHeadersXMLFile(iqstrat.io.ReadHeadersXMLFile.URL);
    stWells = pRead.Process(cmn.cmnStruct.SWD_WELLS);
  }

  /* ===================================================================== *
   * ----------------------- READ SENSOR METHOD -------------------------- *
   * ===================================================================== */

  /** Method readSensors()
   *  <p> This method will read the Sensor information from the KGS DB
   */

  private void readSensors()
  {
    ReadSensorXMLFile pRead = new ReadSensorXMLFile(quake.io.ReadSensorXMLFile.URL);
    stSensor = pRead.Process(cmn.cmnStruct.GET_SENSORS);
  }

  /* ===================================================================== *
   * ------------------------- READ TRS DATA ----------------------------- *
   * ===================================================================== */

  /** Method readGridLines()
   *  <p> This method will retrieve the Grid Lines for a specific County.
   */

  private void readGridLines()
  {
	String                 saCounty[] = { "harper", "sumner", "kingman", "sedgwick" };
    utilLandGridListStruct st         = null;
    utilLandGridFile       pGrid      = null;

    for (int i=0; i<saCounty.length; i++)
    {
      pGrid = new  utilLandGridFile(
                         util.utilFileIO.URL_FILE,
                         cmn.cmnStruct.LANDGRID,
                         saCounty[i].toLowerCase()+".dat");
      st = pGrid.getLandGridList();

      stLandGrid = util.utilLandGridUtility.addCounty(stLandGrid, st);

      if (st != null)
        st.delete();
      st = null;
	}

    if (pGrid != null)
      pGrid.delete();
    pGrid = null;
  }

  /* ===================================================================== *
   * ----------------------- DISPLAY PLOT FRAME -------------------------- *
   * ===================================================================== */

  /** Method display_Mag_Date_Plot()
   * <p> This method will display the Magnitude versus Date Plot Frame.
   * @param st = Kansas Earthquake Seismic Data Structure
   */

  private void display_Mag_Date_Plot(quakeSeisStruct st)
  {
	if (st != null)
	{
      if (frame != null)
        frame.close();
      frame    = null;

      frame = new quakePlotTimeMagDataFrame(
                quake.quakeSeisUtility.copy( st ) );

      if (st != null)
        st.delete();
      st = null;
	}
  }

  /** Method display_3D_Plot()
   * <p> This method will display the Location 3D Plot Frames.
   * @param st = Kansas Earthquake Seismic Data Structure
   */

  private void display_3D_Plot(quakeSeisStruct st)
  {
	if (st != null)
	{
      if (frame3D != null)
        frame3D.close();
      frame3D    = null;

      frame3D = new quake3DPlotControlFrame(
                  quake.quakeSeisUtility.copy( st ),
                  iqstrat.iqstratHeadersUtility.copyList( stWells ),
                  quake.quakeSensorsUtility.copy( stSensor ),
                  util.utilLandGridUtility.copyList( stLandGrid ));

      if (st != null)
        st.delete();
      st = null;
	}
  }

  /** Method display_3D_Animation()
   * <p> This method will display the Location 3D Animation Plot Frame.
   * @param st = Kansas Earthquake Seismic Data Structure
   */

  private void display_3D_Animation(quakeSeisStruct st)
  {
	if (st != null)
	{
      if (frame3DMovie != null)
        frame3DMovie.close();
      frame3DMovie    = null;

      frame3DMovie = new quakeAnimationFrame(
                      quake.quakeSeisUtility.copy( st ),
                      iqstrat.iqstratHeadersUtility.copyList( stWells ),
                      quake.quakeSensorsUtility.copy( stSensor ),
                      util.utilLandGridUtility.copyList( stLandGrid ));

      if (st != null)
        st.delete();
      st = null;
	}
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btn)
    {
      readWells();
	  readData();
      readArrayData();
      readSensors();
      readGridLines();

      // Copy contents of quake Data List Structure to the
      // quake Seismic Data Structure for plotting.
      // _CENTRAL is the Time Zone to use.

      quakeSeisStruct st = quake.quakeDataUtility.quakeGetSeismic(
		                     quake.quakeDataUtility._CENTRAL, stList);
      st = quake.quakeSeisUtility.bubbleSort(st);
//      quake.quakeSeisUtility.print(st);

      switch (iType)
      {
		case _MAG_DATE:
		  display_Mag_Date_Plot( st );
		  break;

		case _3D_PLOT:
		  display_3D_Plot( st );
		  break;

		case _ANIMATE:
		  display_3D_Animation( st );
		  break;
	  }
	}

    if (event.getSource() == btnExit)
    {
	  close();
	}
  }
}


/*
 *  @version 1.1 08/03/2015
 *  @author  John Victorine
 */
